# -*- coding: utf-8 -*-
# flake8: noqa

import os
import re
import shutil
import tempfile
import unittest

from mock import Mock

import tala.nl.gf.resource
import tala.ddd.schemas
from tala.model.ddd import DDD
from tala.ddd.ddd_py_compiler import DddPyCompiler
from tala.ddd.ddd_xml_compiler import DddXmlCompiler, ViolatesSchemaException
from tala.ddd.services.constants import UNDEFINED_SERVICE_ACTION_FAILURE
from tala.ddd.parser import Parser
from tala.ddd.services.service_interface import ServiceInterface, ServiceActionInterface, DeviceModuleTarget, ServiceParameter, ActionFailureReason, ServiceValidatorInterface
from tala.model.device import DeviceAction
from tala.model.domain import Domain
from tala.model.ontology import Ontology
from tala.model.plan import Plan
from tala.nl.gf.auto_generator import AutoGenerator, UnexpectedParameter, InvalidSortOfBackgroundPredicateException
from tala.nl.gf.grammar_entry_types import Node, Constants

UNKNOWN_CATEGORY = "Unknown;\n"
UNKNOWN_FUNCTION = "unknown_string : Unknown -> Sort_string;\n"
MK_UNKNOWN_FUNCTION = "mkUnknown : String -> Unknown;\n"

UNKNOWN_LINCAT = "Unknown = SS"
UNKNOWN_LINERARIZATION = """unknown_string unknown = ss ("\\"" ++ unknown.s ++ "\\"");\n"""
MK_UNKNOWN_LINEARIZATION = "mkUnknown string = ss string.s;\n"

UNKNOWN_NL_LINEARIZATION = """unknown_string unknown = unknown;\n"""
MK_UNKNOWN_NL_LINEARIZATION = """mkUnknown string = string;\n"""


class AutoGeneratorTestCase(unittest.TestCase):
    def setUp(self):
        class MockupDevice:
            actions = {}
            validities = []

        self._device_class = MockupDevice
        self._grammar = Node(Constants.GRAMMAR)
        self._grammar_compiler = DddPyCompiler()
        self._ontology = None
        self._domain = None
        self._ddd_name = "mockup_ddd"

        self._service_actions = []
        self._service_validators = []
        self.mock_service_target = self._create_mock_service_target()

    def given_ontology(self, *args, **kwargs):
        self._create_ontology(*args, **kwargs)

    def _create_ontology(self, sorts=None, predicates=None, actions=None, individuals=None):
        if sorts is None:
            sorts = {
                "city": {},
                "passenger_type": {},
                "keyword": {
                    "dynamic": True
                },
                "membership": {},
            }

        if predicates is None:
            predicates = {
                "dest_city": "city",
                "dest_city_type": {
                    "sort": "city",
                    "feature_of": "dest_city"
                },
                "dept_city": "city",
                "distance": "integer",
                "price": "integer",
                "day_to_set": "integer",
                "hour_to_set": "integer",
                "minute_to_set": "integer",
                "passenger_type_to_add": "passenger_type",
                "passenger_quantity_to_add": "integer",
                "qualified_for_membership": "boolean",
                "frequent_flyer_points": "integer",
                "location": "string",
                "next_membership_level": "membership",
                "next_membership_points": "integer",
                "comment": "string",
                "attraction_information": "string",
                "attraction": "string",
                "flight_departure": "datetime",
            }

        if actions is None:
            actions = set([
                "top",
                "make_reservation",
                "settings",
                "set_time",
            ])

        if individuals is None:
            individuals = {
                "new_york": "city",
            }

        class MockupOntology:
            pass

        MockupOntology.sorts = sorts
        MockupOntology.predicates = predicates
        MockupOntology.actions = actions
        MockupOntology.individuals = individuals
        self._ontology_class = MockupOntology

        ontology_args = DddPyCompiler().compile_ontology(self._ontology_class)
        self._ontology = Ontology(**ontology_args)
        self._parser = Parser(self._ddd_name, self._ontology)

    def _create_mock_service_target(self):
        mock_target = Mock(spec=DeviceModuleTarget)
        mock_target.is_frontend = False
        return mock_target

    def given_compulsory_grammar_for_app(self, exclude=None):
        entries = [
            (Constants.ACTION, {
                "name": "top"
            }, [Node(Constants.ITEM, {}, ["start view"])]),
            (Constants.ACTION, {
                "name": "up"
            }, [Node(Constants.ITEM, {}, ["go back"])]),
            (Constants.ACTION, {
                "name": "make_reservation"
            }, [Node(Constants.ITEM, {}, ["make a reservation"])]),
            (Constants.ACTION, {
                "name": "settings"
            }, [Node(Constants.ITEM, {}, ["settings"])]),
            (Constants.ACTION, {
                "name": "set_time"
            }, [Node(Constants.ITEM, {}, ["set the time"])]),
            (Constants.INDIVIDUAL, {
                "name": "new_york"
            }, [Node(Constants.ITEM, {}, ["New York"])]),
        ]
        for node_type, parameters, children in entries:
            if exclude != (node_type, parameters):
                self._grammar.add_child(Node(node_type, parameters, children))

    def given_grammar(self, added_nodes):
        for node in added_nodes:
            self._grammar.add_child(node)

    def given_form(self, form):
        self._form = form

    def given_generator(self, language_code="eng"):
        self._create_generator(language_code)

    def when_generating(self, language_code="eng"):
        self._create_generator(language_code)
        self.generator.generate(language_code)

    def _then_no_exception_is_raised(self):
        pass

    def _create_generator(self, language_code):
        self._create_mock_ddd()

        def get_grammar(language_code):
            return self._grammar

        self.generator = MockAutoGenerator(self._ddd)
        self.generator._load_and_compile_grammar_entries = get_grammar
        self.generator.warning = ""
        self.generator._grammar_compiler = self._grammar_compiler

    def _create_mock_ddd(self):
        def create_mocked_service_interface():
            mocked_interface = Mock(spec=ServiceInterface)
            mocked_interface.actions = self._service_actions
            mocked_interface.queries = []
            mocked_interface.validators = self._service_validators
            mocked_interface.entity_recognizers = []
            return mocked_interface

        self._ensure_ontology_exists()
        self._ensure_domain_exists()

        self._ddd = Mock(spec=DDD)
        self._ddd.domain = self._domain
        self._ddd.service_interface = create_mocked_service_interface()
        self._ddd.name = "MockupDdd"
        self._ddd.ontology = self._ontology

    def given_service_interface_has_action(self, action):
        self._service_actions.append(action)

    def given_service_interface_has_validity(self, validator):
        self._service_validators.append(validator)

    def _ensure_ontology_exists(self):
        if self._ontology is None:
            self._create_ontology()

    def _ensure_domain_exists(self):
        if self._domain is None:
            self._domain = Domain(self._ddd_name, "mockup_domain", self._ontology)

    def given_py_grammar(self):
        self._grammar_compiler = DddPyCompiler()

    def given_xml_grammar(self):
        self._grammar_compiler = DddXmlCompiler()

    def expect_warning(self, expected_warning):
        self.assertEquals(expected_warning, self.generator.warning.strip())

    def expect_no_warning(self):
        self.assertEquals("", self.generator.warning)

    def assert_abstract(self, expected):
        self.assertEquals(expected, self.generator._abstract_gf_content.getvalue())

    def assert_abstract_contains(self, expected):
        self.assertIn(expected, self.generator._abstract_gf_content.getvalue())

    def assert_abstract_contains_function(self, expected):
        actual_functions_section = self._get_section(self.generator._abstract_gf_content.getvalue(), "fun")
        self.assertIn(expected, actual_functions_section)

    def assert_abstract_not_contains_function(self, expected):
        actual_functions_section = self._get_section(self.generator._abstract_gf_content.getvalue(), "fun")
        self.assertNotIn(expected, actual_functions_section)

    def _get_section(self, gf_content, start_line, end_line="}\n"):
        match = re.search("%s\n(.*)%s" % (start_line, end_line), gf_content, re.DOTALL)
        if match:
            return match.group(1)
        else:
            self.fail(
                "failed to find section with start_line=%r, end_line=%r in %r" % (start_line, end_line, gf_content)
            )

    def assert_abstract_contains_category(self, expected):
        actual_categories_section = self._get_section(
            self.generator._abstract_gf_content.getvalue(), start_line="cat", end_line="fun"
        )
        self.assertIn(expected, actual_categories_section)

    def assert_abstract_not_contains_category(self, expected):
        actual_categories_section = self._get_section(
            self.generator._abstract_gf_content.getvalue(), start_line="cat", end_line="fun"
        )
        self.assertNotIn(expected, actual_categories_section)

    def assert_semantic_contains_lincat(self, expected):
        actual_lincat_section = self._get_section(
            self.generator._semantic_gf_content.getvalue(), start_line="lincat", end_line="lin"
        )
        self.assertIn(expected, actual_lincat_section)

    def assert_semantic_not_contains_lincat(self, expected):
        actual_lincat_section = self._get_section(
            self.generator._semantic_gf_content.getvalue(), start_line="lincat", end_line="lin"
        )
        self.assertNotIn(expected, actual_lincat_section)

    def assert_semantic(self, expected):
        self.assertEquals(expected, self.generator._semantic_gf_content.getvalue())

    def assert_semantic_contains(self, expected):
        self.assertIn(expected, self.generator._semantic_gf_content.getvalue())

    def assert_semantic_contains_linearization(self, expected):
        actual_linearizations_section = self._get_section(self.generator._semantic_gf_content.getvalue(), "lin")
        self.assertIn(expected, actual_linearizations_section)

    def assert_semantic_not_contains_linearization(self, expected):
        actual_linearizations_section = self._get_section(self.generator._semantic_gf_content.getvalue(), "lin")
        self.assertNotIn(expected, actual_linearizations_section)

    def assert_natural_language(self, expected):
        self.assertEquals(expected, self.generator._natural_language_gf_content.getvalue())

    def assert_natural_language_contains(self, expected):
        self.assertIn(expected, self.generator._natural_language_gf_content.getvalue())

    def assert_natural_language_contains_linearization(self, expected):
        actual_linearizations_section = self._get_section(self.generator._natural_language_gf_content.getvalue(), "lin")
        self.assertIn(expected, actual_linearizations_section)

    def assert_natural_language_not_contains_linearization(self, expected):
        actual_linearizations_section = self._get_section(self.generator._natural_language_gf_content.getvalue(), "lin")
        self.assertNotIn(expected, actual_linearizations_section)

    def get_predicate(self, predicate_name):
        return self.generator._ontology.get_predicate(predicate_name)

    def when_generating_action_content(self, action_name):
        action_node = Node(Constants.ACTION, {"name": action_name})
        self._grammar.add_child(action_node)
        if self._form.type == Constants.VP:
            action_node.add_child(Node(Constants.ITEM, {}, [self._form]))
        else:
            action_node.add_child(self._form)
        self.generator._grammar = self._grammar
        self.generator._generate_action_content(action_name)

    def when_generating_predicate_content(self, predicate_name):
        self._grammar.add_child(Node(Constants.PREDICATE, {"name": predicate_name}, [self._form]))
        self.generator._grammar = self._grammar
        self.generator._generate_predicate_content(predicate_name)

    def assert_abstract_begins_with(self, expected_beginning):
        self._assert_begins_with(expected_beginning, self.generator._abstract_gf_content.getvalue())

    def assert_semantic_begins_with(self, expected_beginning):
        self._assert_begins_with(expected_beginning, self.generator._semantic_gf_content.getvalue())

    def assert_natural_language_begins_with(self, expected_beginning):
        self._assert_begins_with(expected_beginning, self.generator._natural_language_gf_content.getvalue())

    def _assert_begins_with(self, expected_beginning, actual_string):
        self.assertTrue(
            actual_string.startswith(expected_beginning),
            "expected %r to begin with %r" % (actual_string, expected_beginning)
        )


class HeaderTestCase(AutoGeneratorTestCase):
    def test_header(self):
        self.given_grammar([])
        self.when_generating()
        self.assert_abstract_begins_with("abstract MockupDdd = TDM, Integers ** {")
        self.assert_semantic_begins_with(
            "concrete MockupDdd_sem of MockupDdd = TDM_sem, Integers_sem ** open Utils_sem in {"
        )
        self.assert_natural_language_begins_with(
            "concrete MockupDdd_eng of MockupDdd = TDM_eng, Integers_eng ** open Utils_eng, Prelude in {"
        )


class CategoriesTestCase(AutoGeneratorTestCase):
    def test_sortal_category(self):
        self.given_ontology(sorts={"city": {}}, predicates={}, actions=set([]), individuals={})
        self.given_grammar([])
        self.when_generating()
        self.assert_abstract_contains_category("Sort_city;\n")

    def test_predicate_category(self):
        self.given_ontology(sorts={"city": {}}, predicates={"dest_city": "city"}, actions=set([]), individuals={})
        self.given_grammar([])
        self.when_generating()
        self.assert_abstract_contains_category("Predicate_dest_city;\n")


class ActionTestCase(AutoGeneratorTestCase):
    def test_action_base_case(self):
        self.given_grammar([
            Node(Constants.ACTION, {"name": "settings"}, [Node(Constants.ITEM, {}, [u"inställningar"])])
        ])
        self.when_generating()
        self.assert_abstract_contains_function('settings : VpAction;')
        self.assert_semantic_contains_linearization('settings = pp "settings";')
        self.assert_natural_language_contains_linearization(
            u'settings = (mkverb "inställningar" "inställningar" "inställningar");'
        )

    def test_vp_action_as_single_variant(self):
        self.given_grammar([
            Node(
                Constants.ACTION, {"name": "set_time"}, [
                    Node(
                        Constants.ITEM, {}, [
                            Node(
                                Constants.VP, {}, [
                                    Node(Constants.INFINITIVE, {}, [u"ställ in"]),
                                    Node(Constants.IMPERATIVE, {}, [u"ställ in"]),
                                    Node(Constants.ING_FORM, {}, [u"ställer in"]),
                                    Node(Constants.OBJECT, {}, ["klockan"])
                                ]
                            )
                        ]
                    )
                ]
            )
        ])
        self.when_generating()
        self.assert_abstract_contains_function('set_time : VpAction;')
        self.assert_semantic_contains_linearization('set_time = pp "set_time";')
        self.assert_natural_language_contains_linearization(
            u'set_time = (mkverb "ställ in" "ställ in" "ställer in" "klockan");'
        )

    def test_vp_action_among_multiple_variants(self):
        self.given_grammar([
            Node(
                Constants.ACTION, {"name": "set_time"}, [
                    Node(
                        Constants.ONE_OF, {}, [
                            Node(
                                Constants.ITEM, {}, [
                                    Node(
                                        Constants.VP, {}, [
                                            Node(Constants.INFINITIVE, {}, [u"ställ in"]),
                                            Node(Constants.IMPERATIVE, {}, [u"ställ in"]),
                                            Node(Constants.ING_FORM, {}, [u"ställer in"]),
                                            Node(Constants.OBJECT, {}, ["klockan"])
                                        ]
                                    )
                                ]
                            ),
                            Node(Constants.ITEM, {}, [u"sätt klockan"])
                        ]
                    )
                ]
            )
        ])
        self.when_generating()
        self.assert_abstract_contains_function('set_time : VpAction;')
        self.assert_semantic_contains_linearization('set_time = pp "set_time";')
        self.assert_natural_language_contains_linearization(
            u'set_time = (mkverb "ställ in" "ställ in" "ställer in" "klockan"|mkverb "sätt klockan" "sätt klockan" "sätt klockan");'
        )

    def test_np_action_with_explicit_definite(self):
        self.given_grammar([
            Node(
                Constants.ACTION, {"name": "settings"}, [
                    Node(
                        Constants.NP, {}, [
                            Node(Constants.INDEFINITE, {}, [u"inställningar"]),
                            Node(Constants.DEFINITE, {}, [u"inställningarna"])
                        ]
                    )
                ]
            )
        ])
        self.when_generating()
        self.assert_abstract_contains_function('settings : NpAction;')
        self.assert_semantic_contains_linearization('settings = pp "settings";')
        self.assert_natural_language_contains_linearization(u'settings = (mkdef "inställningar" "inställningarna");')

    def test_np_action_with_implicit_definite(self):
        self.given_grammar([
            Node(
                Constants.ACTION, {"name": "settings"},
                [Node(Constants.NP, {}, [Node(Constants.INDEFINITE, {}, [u"preferences"])])]
            )
        ])
        self.when_generating()
        self.assert_abstract_contains_function('settings : NpAction;')
        self.assert_semantic_contains_linearization('settings = pp "settings";')
        self.assert_natural_language_contains_linearization('settings = (mkdef "preferences");')

    def test_singular_feminine_np_action(self):
        self.given_grammar([
            Node(
                Constants.ACTION, {
                    "name": "settings",
                    "number": tala.nl.gf.resource.SINGULAR,
                    "gender": tala.nl.gf.resource.FEMININE
                }, [Node(Constants.NP, {}, [Node(Constants.INDEFINITE, {}, [u"impostazione"])])]
            )
        ])
        self.when_generating()
        self.assert_abstract_contains_function('settings : NpActionSF;')
        self.assert_semantic_contains_linearization('settings = pp "settings";')
        self.assert_natural_language_contains_linearization('settings = (mkdef "impostazione");')

    def test_singular_masculine_np_action(self):
        self.given_grammar([
            Node(
                Constants.ACTION, {
                    "name": "top",
                    "number": tala.nl.gf.resource.SINGULAR,
                    "gender": tala.nl.gf.resource.MASCULINE
                }, [Node(Constants.NP, {}, [Node(Constants.INDEFINITE, {}, [u"menu principale"])])]
            )
        ])
        self.when_generating()
        self.assert_abstract_contains_function('top : NpActionSM;')
        self.assert_semantic_contains_linearization('top = pp "top";')
        self.assert_natural_language_contains_linearization('top = (mkdef "menu principale");')

    def test_plural_feminine_np_action(self):
        self.given_grammar([
            Node(
                Constants.ACTION, {
                    "name": "settings",
                    "number": tala.nl.gf.resource.PLURAL,
                    "gender": tala.nl.gf.resource.FEMININE
                }, [Node(Constants.NP, {}, [Node(Constants.INDEFINITE, {}, [u"impostazioni"])])]
            )
        ])
        self.when_generating()
        self.assert_abstract_contains_function('settings : NpActionPF;')
        self.assert_semantic_contains_linearization('settings = pp "settings";')
        self.assert_natural_language_contains_linearization('settings = (mkdef "impostazioni");')

    def test_plural_masculine_np_action(self):
        self.given_grammar([
            Node(
                Constants.ACTION, {
                    "name": "top",
                    "number": tala.nl.gf.resource.PLURAL,
                    "gender": tala.nl.gf.resource.MASCULINE
                }, [Node(Constants.NP, {}, [Node(Constants.INDEFINITE, {}, [u"menu principali"])])]
            )
        ])
        self.when_generating()
        self.assert_abstract_contains_function('top : NpActionPM;')
        self.assert_semantic_contains_linearization('top = pp "top";')
        self.assert_natural_language_contains_linearization('top = (mkdef "menu principali");')

    def test_np_action_among_multiple_variants(self):
        self.given_grammar([
            Node(
                Constants.ACTION, {"name": "settings"}, [
                    Node(
                        Constants.ONE_OF, {}, [
                            Node(Constants.ITEM, {}, ["options"]),
                            Node(
                                Constants.ITEM, {},
                                [Node(Constants.NP, {}, [Node(Constants.INDEFINITE, {}, [u"preferences"])])]
                            )
                        ]
                    )
                ]
            )
        ])
        self.when_generating()
        self.assert_abstract_contains_function('settings : NpAction;')
        self.assert_semantic_contains_linearization('settings = pp "settings";')
        self.assert_natural_language_contains_linearization('settings = (mkdef "preferences");')
        self.assert_natural_language_contains_linearization('settings_request = ss (("options"));')


class PredicateTestCase(AutoGeneratorTestCase):
    def test_predicate_base_case(self):
        self.given_grammar([
            Node(Constants.PREDICATE, {"name": "price"}, [Node(Constants.ITEM, {}, ["price information"])])
        ])
        self.when_generating()
        self.assert_abstract_contains_function('price : Predicate;')
        self.assert_semantic_contains_linearization('price = pp "price";')
        self.assert_natural_language_contains_linearization('price = ss (("price information"));')


class TitleTestCase(AutoGeneratorTestCase):
    def test_issue_title(self):
        self.given_grammar([
            Node(Constants.ISSUE_TITLE, {"predicate": "price"}, [Node(Constants.ITEM, {}, ["Price Information"])])
        ])
        self.when_generating()
        self.assert_abstract_contains_function('price_title : SysTitle;')
        self.assert_semantic_contains_linearization('price_title = issueTitle price;')
        self.assert_natural_language_contains_linearization('price_title = ss "Price Information";')

    def test_action_title(self):
        self.given_compulsory_grammar_for_app()
        self.given_grammar([Node(Constants.ACTION_TITLE, {"action": "top"}, [Node(Constants.ITEM, {}, ["Main Menu"])])])
        self.when_generating()
        self.assert_abstract_contains_function('top_title : SysTitle;')
        self.assert_semantic_contains_linearization('top_title = actionTitle (pp "top");')
        self.assert_natural_language_contains_linearization('top_title = ss "Main Menu";')


class test_parameterized_sys_form_answer(AutoGeneratorTestCase):
    def test_in_middle(self):
        self.given_form(
            Node(
                Constants.ITEM, {},
                ["cannot set the hour to ",
                 Node(Constants.SLOT, {"predicate": "hour_to_set"}), "."]
            )
        )
        self.given_generator()
        self.when_called()
        self.then_result_is('"cannot set the hour to " ++ hour_to_set.alt ++ "."')

    def test_at_start(self):
        self.given_form(
            Node(Constants.ITEM, {}, [Node(Constants.SLOT, {"predicate": "hour_to_set"}), " is not an OK hour."])
        )
        self.given_generator()
        self.when_called()
        self.then_result_is('hour_to_set.alt ++ " is not an OK hour."')

    def test_at_end(self):
        self.given_form(
            Node(Constants.ITEM, {}, ["cannot set the hour to ",
                                      Node(Constants.SLOT, {"predicate": "hour_to_set"})])
        )
        self.given_generator()
        self.when_called()
        self.then_result_is('"cannot set the hour to " ++ hour_to_set.alt')

    def test_two_parameters(self):
        self.given_form(
            Node(
                Constants.ITEM, {}, [
                    "cannot set the time to ",
                    Node(Constants.SLOT, {"predicate": "hour_to_set"}), " ",
                    Node(Constants.SLOT, {"predicate": "minute_to_set"})
                ]
            )
        )
        self.given_generator()
        self.when_called()
        self.then_result_is('"cannot set the time to " ++ hour_to_set.alt ++ " " ++ minute_to_set.alt')

    def test_two_parameters_switched_order(self):
        self.given_form(
            Node(
                Constants.ITEM, {}, [
                    "cannot set the time to ",
                    Node(Constants.SLOT, {"predicate": "minute_to_set"}), " past ",
                    Node(Constants.SLOT, {"predicate": "hour_to_set"})
                ]
            )
        )
        self.given_generator()
        self.when_called()
        self.then_result_is('"cannot set the time to " ++ minute_to_set.alt ++ " past " ++ hour_to_set.alt')

    def test_in_middle_unicode(self):
        self.given_form(
            Node(
                Constants.ITEM, {},
                [u"kan inte ställa timmen till ",
                 Node(Constants.SLOT, {"predicate": "hour_to_set"}), "."]
            )
        )
        self.given_generator()
        self.when_called()
        self.then_result_is(u'"kan inte ställa timmen till " ++ hour_to_set.alt ++ "."')

    def when_called(self):
        self._actual_result = self.generator._parameterized_sys_form(self._form)

    def then_result_is(self, expected_gf_content):
        self.assertEquals(expected_gf_content, self._actual_result)


class UsrRequestTest(AutoGeneratorTestCase):
    def _create_mock_ddd(self):
        self._ensure_ontology_exists()
        self._ensure_domain_exists()

        self._ddd = Mock(spec=DDD)
        self._ddd.domain = self._domain
        self._ddd.name = "MockupDdd"
        self._ddd.ontology = self._ontology

    def test_two_parameters(self):
        self.given_form(
            Node(
                Constants.ITEM, {}, [
                    "set the time to ",
                    Node(Constants.SLOT, {"predicate": "hour_to_set"}), " ",
                    Node(Constants.SLOT, {"predicate": "minute_to_set"})
                ]
            )
        )
        self.given_generator()
        self.when_generating_action_content("set_time")
        self.assert_abstract_contains(
            'set_time_request_1 : Predicate_hour_to_set -> Predicate_minute_to_set -> UsrRequest;'
        )
        self.assert_semantic_contains(
            'set_time_request_1 hour_to_set minute_to_set = request (pp "set_time") hour_to_set minute_to_set;\n'
        )
        self.assert_natural_language(
            'set_time_request_1 hour_to_set minute_to_set = ss ("set the time to " ++ hour_to_set.s ++ " " ++ minute_to_set.s);\n'
        )

    def test_two_parameters_unicode(self):
        self.given_form(
            Node(
                Constants.ITEM, {}, [
                    u"ställ klockan på ",
                    Node(Constants.SLOT, {"predicate": "hour_to_set"}), " ",
                    Node(Constants.SLOT, {"predicate": "minute_to_set"})
                ]
            )
        )
        self.given_generator()
        self.when_generating_action_content("set_time")
        self.assert_abstract_contains(
            'set_time_request_1 : Predicate_hour_to_set -> Predicate_minute_to_set -> UsrRequest;'
        )
        self.assert_semantic_contains(
            'set_time_request_1 hour_to_set minute_to_set = request (pp "set_time") hour_to_set minute_to_set;\n'
        )
        self.assert_natural_language(
            u'set_time_request_1 hour_to_set minute_to_set = ss ("ställ klockan på " ++ hour_to_set.s ++ " " ++ minute_to_set.s);\n'
        )

    def test_action_with_literal_parameter(self):
        self.given_form(
            Node(
                Constants.ITEM, {},
                ["leave a comment to the housing owner that ",
                 Node(Constants.SLOT, {"predicate": "comment"})]
            )
        )
        self.given_generator()
        self.when_generating_action_content("leave_comment")
        self.assert_abstract_contains(
            'leave_comment : VpAction;\n'
            'leave_comment_request_1 : Predicate_comment -> UsrRequest;\n'
        )
        self.assert_semantic_contains(
            'leave_comment = pp "leave_comment";\n'
            'leave_comment_request_1 comment = request (pp "leave_comment") comment;\n'
        )
        self.assert_natural_language(
            'leave_comment_request_1 comment = ss ("leave a comment to the housing owner that " ++ comment.s);\n'
        )

    def test_action_with_literal_and_non_literal_parameter(self):
        self.given_form(
            Node(
                Constants.ITEM, {}, [
                    "I will arrive at ",
                    Node(Constants.SLOT, {"predicate": "hour_to_set"}),
                    "and leave a comment to the housing owner that ",
                    Node(Constants.SLOT, {"predicate": "comment"})
                ]
            )
        )
        self.given_generator()
        self.when_generating_action_content("leave_comment")
        self.assert_abstract_contains(
            'leave_comment : VpAction;\n'
            'leave_comment_request_1 : Predicate_hour_to_set -> Predicate_comment -> UsrRequest;\n'
        )
        self.assert_semantic_contains(
            'leave_comment = pp "leave_comment";\n'
            'leave_comment_request_1 hour_to_set comment = request (pp "leave_comment") hour_to_set comment;\n'
        )
        self.assert_natural_language(
            'leave_comment_request_1 hour_to_set comment = ss ("I will arrive at " ++ hour_to_set.s ++ "and leave a comment to the housing owner that " ++ comment.s);\n'
        )


class ValidityTest(AutoGeneratorTestCase):
    def test_all_parameters_in_grammar(self):
        self.given_service_interface_has_validity(
            ServiceValidatorInterface(
                "CityValidity",
                self.mock_service_target,
                parameters=[
                    ServiceParameter("dept_city", is_optional=True),
                    ServiceParameter("dest_city", is_optional=True),
                ]
            )
        )
        self.given_grammar([
            Node(
                Constants.VALIDITY, {"name": "CityValidity"}, [
                    Node(
                        Constants.ITEM, {}, [
                            "cannot go from ",
                            Node(Constants.SLOT, {"predicate": "dept_city"}), " to ",
                            Node(Constants.SLOT, {"predicate": "dest_city"})
                        ]
                    )
                ]
            )
        ])
        self.when_generating()
        self.assert_abstract_contains_function("CityValidity_1 : SysAnswer -> SysAnswer -> SysICM;")
        self.assert_semantic_contains_linearization(
            'CityValidity_1 dept_city dest_city = rejectICM (set (list dept_city dest_city)) "CityValidity";'
        )
        self.assert_natural_language_contains_linearization(
            'CityValidity_1 dept_city dest_city = ss ("cannot go from " ++ dept_city.alt ++ " to " ++ dest_city.alt);'
        )

    def test_honour_parameter_order_in_grammar(self):
        self.given_service_interface_has_validity(
            ServiceValidatorInterface(
                "CityValidity",
                self.mock_service_target,
                parameters=[
                    ServiceParameter("dept_city", is_optional=True),
                    ServiceParameter("dest_city", is_optional=True),
                ]
            )
        )
        self.given_grammar([
            Node(
                Constants.VALIDITY, {"name": "CityValidity"}, [
                    Node(
                        Constants.ITEM, {}, [
                            "cannot go to ",
                            Node(Constants.SLOT, {"predicate": "dest_city"}), " from ",
                            Node(Constants.SLOT, {"predicate": "dept_city"})
                        ]
                    )
                ]
            )
        ])
        self.when_generating()
        self.assert_abstract_contains_function("CityValidity_1 : SysAnswer -> SysAnswer -> SysICM;")
        self.assert_semantic_contains_linearization(
            'CityValidity_1 dept_city dest_city = rejectICM (set (list dept_city dest_city)) "CityValidity";'
        )
        self.assert_natural_language_contains_linearization(
            'CityValidity_1 dept_city dest_city = ss ("cannot go to " ++ dest_city.alt ++ " from " ++ dept_city.alt);'
        )

    def test_only_one_optional_parameter_in_grammar(self):
        self.given_service_interface_has_validity(
            ServiceValidatorInterface(
                "CityValidity",
                self.mock_service_target,
                parameters=[
                    ServiceParameter("dept_city", is_optional=True),
                    ServiceParameter("dest_city", is_optional=True),
                ]
            )
        )
        self.given_grammar([
            Node(
                Constants.VALIDITY, {"name": "CityValidity"},
                [Node(Constants.ITEM, {},
                      ["cannot go from ", Node(Constants.SLOT, {"predicate": "dept_city"})])]
            )
        ])
        self.when_generating()
        self.assert_abstract_contains_function("CityValidity_1 : SysAnswer -> SysICM;")
        self.assert_semantic_contains_linearization(
            'CityValidity_1 dept_city = rejectICM (set (list dept_city)) "CityValidity";'
        )
        self.assert_natural_language_contains_linearization(
            'CityValidity_1 dept_city = ss ("cannot go from " ++ dept_city.alt);'
        )

    def test_dont_confuse_parameters(self):
        self.given_service_interface_has_validity(
            ServiceValidatorInterface(
                "CityValidity",
                self.mock_service_target,
                parameters=[
                    ServiceParameter("dept_city", is_optional=True),
                    ServiceParameter("dest_city", is_optional=True),
                ]
            )
        )
        self.given_grammar([
            Node(
                Constants.VALIDITY, {"name": "CityValidity"}, [
                    Node(
                        Constants.ONE_OF, {}, [
                            Node(
                                Constants.ITEM, {},
                                ["cannot go from ", Node(Constants.SLOT, {"predicate": "dept_city"})]
                            ),
                            Node(
                                Constants.ITEM, {},
                                ["cannot go to ", Node(Constants.SLOT, {"predicate": "dest_city"})]
                            )
                        ]
                    )
                ]
            )
        ])
        self.when_generating()
        self.assert_abstract_contains_function("CityValidity_1 : SysAnswer -> SysICM;")
        self.assert_abstract_contains_function("CityValidity_2 : SysAnswer -> SysICM;")
        self.assert_semantic_contains_linearization(
            'CityValidity_1 dept_city = rejectICM (set (list dept_city)) "CityValidity";'
        )
        self.assert_semantic_contains_linearization(
            'CityValidity_2 dest_city = rejectICM (set (list dest_city)) "CityValidity";'
        )
        self.assert_natural_language_contains_linearization(
            'CityValidity_1 dept_city = ss ("cannot go from " ++ dept_city.alt);'
        )
        self.assert_natural_language_contains_linearization(
            'CityValidity_2 dest_city = ss ("cannot go to " ++ dest_city.alt);'
        )

    def test_grammar_may_exclude_some_optional_parameter(self):
        self.given_service_interface_has_validity(
            ServiceValidatorInterface(
                "CityValidity",
                self.mock_service_target,
                parameters=[
                    ServiceParameter("dept_city", is_optional=True),
                    ServiceParameter("dest_city", is_optional=True),
                ]
            )
        )
        self.given_grammar([
            Node(
                Constants.VALIDITY, {"name": "CityValidity"},
                [Node(Constants.ITEM, {},
                      ["cannot go from ", Node(Constants.SLOT, {"predicate": "dept_city"})])]
            )
        ])
        self.when_generating()
        self.assert_abstract_contains_function("CityValidity_2 : SysAnswer -> SysAnswer -> SysICM;")
        self.assert_semantic_contains_linearization(
            'CityValidity_2 dept_city dest_city = rejectICM (set (list dept_city dest_city)) "CityValidity";'
        )
        self.assert_natural_language_contains_linearization(
            'CityValidity_2 dept_city dest_city = ss ("cannot go from " ++ dept_city.alt);'
        )

    def test_grammar_may_exclude_all_optional_parameter(self):
        self.given_service_interface_has_validity(
            ServiceValidatorInterface(
                "CityValidity",
                self.mock_service_target,
                parameters=[
                    ServiceParameter("dept_city", is_optional=True),
                    ServiceParameter("dest_city", is_optional=True),
                ]
            )
        )
        self.given_grammar([
            Node(Constants.VALIDITY, {"name": "CityValidity"}, [Node(Constants.ITEM, {}, ["invalid parameters"])])
        ])
        self.when_generating()
        self.assert_abstract_contains_function("CityValidity_1 : SysAnswer -> SysICM;")
        self.assert_abstract_contains_function("CityValidity_2 : SysAnswer -> SysICM;")
        self.assert_semantic_contains_linearization(
            'CityValidity_1 dept_city = rejectICM (set (list dept_city)) "CityValidity";'
        )
        self.assert_semantic_contains_linearization(
            'CityValidity_2 dest_city = rejectICM (set (list dest_city)) "CityValidity";'
        )
        self.assert_natural_language_contains_linearization('CityValidity_1 dept_city = ss ("invalid parameters");')
        self.assert_natural_language_contains_linearization('CityValidity_2 dest_city = ss ("invalid parameters")')


class MockAutoGenerator(AutoGenerator):
    def _missing_entry(self, reporting_method, *args):
        self.missing_entry_warned = True
        reporting_method(*args)

    def _warn(self, line):
        self.warning += line
        self.warning += '\n'


class ReportTestCase(AutoGeneratorTestCase):
    def test_multiple_parameters_base(self):
        self.given_grammar([
            Node(
                Constants.REPORT_ENDED, {"action": "SetTime"}, [
                    Node(
                        Constants.ITEM, {}, [
                            "the time was set to ",
                            Node(Constants.SLOT, {"predicate": "hour_to_set"}), " ",
                            Node(Constants.SLOT, {"predicate": "minute_to_set"}), "."
                        ]
                    )
                ]
            )
        ])
        self.given_service_interface_has_action(
            ServiceActionInterface(
                "SetTime",
                self.mock_service_target,
                parameters=[
                    ServiceParameter("hour_to_set"),
                    ServiceParameter("minute_to_set"),
                ],
                failure_reasons=[]
            )
        )
        self.given_generator()
        self.when_generating()
        self.assert_abstract_contains_function('report_ended_SetTime_1 : SysAnswer -> SysAnswer -> SysReportEnded;\n')
        self.assert_semantic_contains_linearization(
            'report_ended_SetTime_1 hour_to_set minute_to_set = report_ended "SetTime" (list hour_to_set minute_to_set);\n'
        )
        self.assert_natural_language_contains_linearization(
            'report_ended_SetTime_1 hour_to_set minute_to_set = ss ("the time was set to " ++ hour_to_set.alt ++ " " ++ minute_to_set.alt ++ ".");\n'
        )

    def test_multiple_parameters_with_other_order_in_grammar_entry(self):
        self.given_grammar([
            Node(
                Constants.REPORT_ENDED, {"action": "SetTime"}, [
                    Node(
                        Constants.ITEM, {}, [
                            "the time was set to ",
                            Node(Constants.SLOT, {"predicate": "minute_to_set"}), " past ",
                            Node(Constants.SLOT, {"predicate": "hour_to_set"}), "."
                        ]
                    )
                ]
            )
        ])
        self.given_service_interface_has_action(
            ServiceActionInterface(
                "SetTime",
                self.mock_service_target,
                parameters=[
                    ServiceParameter("hour_to_set"),
                    ServiceParameter("minute_to_set"),
                ],
                failure_reasons=[]
            )
        )
        self.given_generator()
        self.when_generating()
        self.assert_abstract_contains_function('report_ended_SetTime_1 : SysAnswer -> SysAnswer -> SysReportEnded;\n')
        self.assert_semantic_contains_linearization(
            'report_ended_SetTime_1 hour_to_set minute_to_set = report_ended "SetTime" (list hour_to_set minute_to_set);\n'
        )
        self.assert_natural_language_contains_linearization(
            'report_ended_SetTime_1 hour_to_set minute_to_set = ss ("the time was set to " ++ minute_to_set.alt ++ " past " ++ hour_to_set.alt ++ ".");\n'
        )

    def test_multiple_parameters_with_default(self):
        self.given_grammar([
            Node(
                Constants.REPORT_ENDED, {"action": "SetTime"}, [
                    Node(
                        Constants.ITEM, {}, [
                            "the time was set to ",
                            Node(Constants.SLOT, {"predicate": "hour_to_set"}), " ",
                            Node(Constants.SLOT, {"predicate": "minute_to_set"}), "."
                        ]
                    )
                ]
            )
        ])
        self.given_service_interface_has_action(
            ServiceActionInterface(
                "SetTime",
                self.mock_service_target,
                parameters=[
                    ServiceParameter("hour_to_set", is_optional=True),
                    ServiceParameter("minute_to_set", is_optional=True),
                ],
                failure_reasons=[]
            )
        )
        self.given_generator()
        self.when_generating()
        self.assert_abstract_contains_function('report_ended_SetTime_1 : SysAnswer -> SysAnswer -> SysReportEnded;\n')
        self.assert_semantic_contains_linearization(
            'report_ended_SetTime_1 hour_to_set minute_to_set = report_ended "SetTime" (list hour_to_set minute_to_set);\n'
        )
        self.assert_natural_language_contains_linearization(
            'report_ended_SetTime_1 hour_to_set minute_to_set = ss ("the time was set to " ++ hour_to_set.alt ++ " " ++ minute_to_set.alt ++ ".");\n'
        )

    def test_parameters_ignored_in_grammar_entry(self):
        self.given_grammar([
            Node(Constants.REPORT_ENDED, {"action": "SetTime"}, [Node(Constants.ITEM, {}, ["the time was set."])])
        ])
        self.given_service_interface_has_action(
            ServiceActionInterface(
                "SetTime",
                self.mock_service_target,
                parameters=[
                    ServiceParameter("hour_to_set"),
                    ServiceParameter("minute_to_set"),
                ],
                failure_reasons=[]
            )
        )
        self.given_generator()
        self.when_generating()
        self.assert_abstract_contains_function('report_ended_SetTime_1 : SysAnswer -> SysAnswer -> SysReportEnded;\n')
        self.assert_semantic_contains_linearization(
            'report_ended_SetTime_1 hour_to_set minute_to_set = report_ended "SetTime" (list hour_to_set minute_to_set);\n'
        )
        self.assert_natural_language_contains_linearization(
            'report_ended_SetTime_1 hour_to_set minute_to_set = ss ("the time was set.");\n'
        )

    def test_no_parameters(self):
        self.given_grammar([
            Node(Constants.REPORT_ENDED, {"action": "SetTime"}, [Node(Constants.ITEM, {}, ["the time was set."])])
        ])
        self.given_service_interface_has_action(
            ServiceActionInterface("SetTime", self.mock_service_target, parameters=[], failure_reasons=[])
        )
        self.given_generator()
        self.when_generating()
        self.assert_abstract_contains_function('report_ended_SetTime_1 : SysReportEnded;\n')
        self.assert_semantic_contains_linearization('report_ended_SetTime_1 = report_ended "SetTime" (empty_list);\n')
        self.assert_natural_language_contains_linearization('report_ended_SetTime_1 = ss ("the time was set.");\n')

    def test_empty_form(self):
        self.given_grammar([Node(Constants.REPORT_ENDED, {"action": "SetTime"}, [Node(Constants.ITEM, {}, [])])])
        self.given_service_interface_has_action(
            ServiceActionInterface("SetTime", self.mock_service_target, parameters=[], failure_reasons=[])
        )
        self.given_generator()
        self.when_generating()
        self._then_no_exception_is_raised()

    def test_unicode(self):
        self.given_grammar([
            Node(
                Constants.REPORT_ENDED, {"action": "SetTime"}, [Node(Constants.ITEM, {}, [u"klockan har ställts in."])]
            )
        ])
        self.given_service_interface_has_action(
            ServiceActionInterface("SetTime", self.mock_service_target, parameters=[], failure_reasons=[])
        )
        self.given_generator()
        self.when_generating()
        self.assert_abstract_contains_function('report_ended_SetTime_1 : SysReportEnded;\n')
        self.assert_semantic_contains_linearization('report_ended_SetTime_1 = report_ended "SetTime" (empty_list);\n')
        self.assert_natural_language_contains_linearization(
            u'report_ended_SetTime_1 = ss ("klockan har ställts in.");\n'
        )

    def test_reason_and_no_parameters(self):
        self.given_grammar([
            Node(
                Constants.REPORT_FAILED, {
                    "action": "Snooze",
                    "reason": "not_ringing"
                }, [Node(Constants.ITEM, {}, ["the alarm is not ringing."])]
            )
        ])
        self.given_service_interface_has_action(
            ServiceActionInterface(
                "Snooze",
                DeviceModuleTarget(device="MockedDevice"),
                parameters=[],
                failure_reasons=[
                    ActionFailureReason("not_ringing"),
                ]
            )
        )
        self.given_generator()
        self.when_generating()
        self.assert_abstract_contains_function('report_failed_Snooze_not_ringing_2 : SysReportFailed;\n')
        self.assert_semantic_contains_linearization(
            'report_failed_Snooze_not_ringing_2 = report_failed "Snooze" (empty_list) "not_ringing";\n'
        )
        self.assert_natural_language_contains_linearization(
            'report_failed_Snooze_not_ringing_2 = ss ("the alarm is not ringing.");\n'
        )

    def test_reason_and_optional_parameter(self):
        self.given_grammar([
            Node(
                Constants.REPORT_FAILED, {
                    "action": "SetTime",
                    "reason": "invalid_time"
                }, [
                    Node(
                        Constants.ONE_OF, {}, [
                            Node(
                                Constants.ITEM, {}, [
                                    "failed to set the time to ",
                                    Node(Constants.SLOT, {"predicate": "hour_to_set"}), " hours."
                                ]
                            ),
                            Node(
                                Constants.ITEM, {}, [
                                    "failed to set the time to ",
                                    Node(Constants.SLOT, {"predicate": "hour_to_set"}), " ",
                                    Node(Constants.SLOT, {"predicate": "minute_to_set"}), "."
                                ]
                            )
                        ]
                    )
                ]
            )
        ])
        self.given_service_interface_has_action(
            ServiceActionInterface(
                "SetTime",
                DeviceModuleTarget(device="MockedDevice"),
                parameters=[ServiceParameter("hour_to_set"),
                            ServiceParameter("minute_to_set", is_optional=True)],
                failure_reasons=[
                    ActionFailureReason("invalid_time"),
                ]
            )
        )
        self.given_generator()
        self.when_generating()
        self.assert_abstract_contains_function('report_failed_SetTime_invalid_time_3 : SysAnswer -> SysReportFailed;\n')
        self.assert_abstract_contains_function(
            'report_failed_SetTime_invalid_time_4 : SysAnswer -> SysAnswer -> SysReportFailed;\n'
        )
        self.assert_semantic_contains_linearization(
            'report_failed_SetTime_invalid_time_3 hour_to_set = report_failed "SetTime" (list hour_to_set) "invalid_time";\n'
        )
        self.assert_semantic_contains_linearization(
            'report_failed_SetTime_invalid_time_4 hour_to_set minute_to_set = report_failed "SetTime" (list hour_to_set minute_to_set) "invalid_time";\n'
        )
        self.assert_natural_language_contains_linearization(
            'report_failed_SetTime_invalid_time_3 hour_to_set = ss ("failed to set the time to " ++ hour_to_set.alt ++ " hours.");\n'
        )
        self.assert_natural_language_contains_linearization(
            'report_failed_SetTime_invalid_time_4 hour_to_set minute_to_set = ss ("failed to set the time to " ++ hour_to_set.alt ++ " " ++ minute_to_set.alt ++ ".");\n'
        )

    def test_unknown_failure_without_parameters(self):
        class Snooze(DeviceAction):
            PARAMETERS = []

        self.given_grammar([])
        self.given_service_interface_has_action(
            ServiceActionInterface("Snooze", self.mock_service_target, parameters=[], failure_reasons=[])
        )
        self.given_generator()
        self.when_generating()
        self.assert_abstract_contains_function('report_failed_Snooze_undefined_failure_1 : SysReportFailed;\n')
        self.assert_semantic_contains_linearization(
            'report_failed_Snooze_undefined_failure_1 = report_failed "Snooze" (empty_list) "%s";\n' %
            UNDEFINED_SERVICE_ACTION_FAILURE
        )
        self.assert_natural_language_contains_linearization(
            'report_failed_Snooze_undefined_failure_1 = undefined_service_action_failure;\n'
        )

    def test_unknown_failure_with_parameters(self):
        self.given_grammar([
            Node(
                Constants.REPORT_FAILED, {
                    "action": "SetTime",
                    "reason": "invalid_time"
                }, [
                    Node(
                        Constants.ONE_OF, {}, [
                            Node(
                                Constants.ITEM, {}, [
                                    "failed to set the time to ",
                                    Node(Constants.SLOT, {"predicate": "hour_to_set"}), " hours."
                                ]
                            ),
                            Node(
                                Constants.ITEM, {}, [
                                    "failed to set the time to ",
                                    Node(Constants.SLOT, {"predicate": "hour_to_set"}), " ",
                                    Node(Constants.SLOT, {"predicate": "minute_to_set"}), "."
                                ]
                            )
                        ]
                    )
                ]
            )
        ])
        self.given_service_interface_has_action(
            ServiceActionInterface(
                "SetTime",
                self.mock_service_target,
                parameters=[ServiceParameter("hour_to_set"),
                            ServiceParameter("minute_to_set", is_optional=True)],
                failure_reasons=[]
            )
        )
        self.given_generator()
        self.when_generating()
        self.assert_abstract_contains_function(
            'report_failed_SetTime_undefined_failure_1 : SysAnswer -> SysReportFailed;\n'
        )
        self.assert_abstract_contains_function(
            'report_failed_SetTime_undefined_failure_2 : SysAnswer -> SysAnswer -> SysReportFailed;\n'
        )
        self.assert_semantic_contains_linearization(
            'report_failed_SetTime_undefined_failure_1 hour_to_set = report_failed "SetTime" (list hour_to_set) "%s";\n'
            % UNDEFINED_SERVICE_ACTION_FAILURE
        )
        self.assert_semantic_contains_linearization(
            'report_failed_SetTime_undefined_failure_2 hour_to_set minute_to_set = report_failed "SetTime" (list hour_to_set minute_to_set) "%s";\n'
            % UNDEFINED_SERVICE_ACTION_FAILURE
        )
        self.assert_natural_language_contains_linearization(
            'report_failed_SetTime_undefined_failure_1 hour_to_set = undefined_service_action_failure;\n'
        )
        self.assert_natural_language_contains_linearization(
            'report_failed_SetTime_undefined_failure_2 hour_to_set minute_to_set = undefined_service_action_failure;\n'
        )


class VerbPhraseTest(AutoGeneratorTestCase):
    def test_verb_phrase(self):
        self.given_form(
            Node(
                Constants.VP, {}, [
                    Node(Constants.INFINITIVE, {}, ["set"]),
                    Node(Constants.IMPERATIVE, {}, ["set"]),
                    Node(Constants.ING_FORM, {}, ["setting"]),
                    Node(Constants.OBJECT, {}, ["the time"])
                ]
            )
        )
        self.given_generator()
        self.when_generating_action_content("set_time")
        self.assert_abstract_contains("set_time : VpAction;")
        self.assert_semantic_contains('set_time = pp "set_time";')
        self.assert_natural_language_contains('set_time = (mkverb "set" "set" "setting" "the time")')


class WarningsAndErrorsTest(AutoGeneratorTestCase):
    def test_warn_about_missing_entry_for_postconfirmed_action(self):
        self.given_service_interface_has_action(
            ServiceActionInterface("SetAlarm", self.mock_service_target, parameters=[], failure_reasons=[])
        )
        self.given_action_is_postconfirmed("SetAlarm")
        self.given_compulsory_grammar_for_app()
        self.when_generating()
        self.expect_warning(
            'How does the system report that the service action SetAlarm ended? Specify the utterance:\n\n'
            'SetAlarm_ended = "performed SetAlarm."'
        )

    def test_warn_about_missing_entry_for_preconfirmation(self):
        self.given_service_interface_has_action(
            ServiceActionInterface("SetAlarm", self.mock_service_target, parameters=[], failure_reasons=[])
        )
        self.given_action_is_preconfirmed("SetAlarm")
        self.given_compulsory_grammar_for_app()
        self.when_generating()
        self.expect_warning(
            'How does the system ask the user to confirm the service action SetAlarm, before performing the action? The entry is used in questions such as "Do you want to X?" where X is the grammar entry. Example:\n\n'
            'SetAlarm_preconfirm = "perform SetAlarm."'
        )

    def test_warn_about_missing_entry_for_prereport(self):
        self.given_service_interface_has_action(
            ServiceActionInterface("SetAlarm", self.mock_service_target, parameters=[], failure_reasons=[])
        )
        self.given_action_is_prereported("SetAlarm")
        self.given_compulsory_grammar_for_app()
        self.when_generating()
        self.expect_warning(
            'How does the system give positive feedback about the service action SetAlarm, before performing the action? Specify the utterance:\n\nSetAlarm_prereport = "performing SetAlarm."'
        )

    def test_not_warn_about_missing_entry_if_entry_exists(self):
        self.given_service_interface_has_action(
            ServiceActionInterface("AlarmRings", self.mock_service_target, parameters=[], failure_reasons=[])
        )
        self.given_action_is_postconfirmed("AlarmRings")
        self.given_compulsory_grammar_for_app()
        self.given_grammar([
            Node(Constants.REPORT_ENDED, {"action": "AlarmRings"}, [Node(Constants.ITEM, {}, ["Beep beep!"])])
        ])
        self.when_generating()
        self.expect_no_warning()

    def test_not_warn_about_missing_entry_if_no_postconfirmation(self):
        self.given_service_interface_has_action(
            ServiceActionInterface("AlarmRings", self.mock_service_target, parameters=[], failure_reasons=[])
        )
        self.given_action_is_not_postconfirmed("AlarmRings")
        self.given_compulsory_grammar_for_app()
        self.when_generating()
        self.expect_no_warning()

    def test_generates_report_ended_if_entry_exists_even_without_postconfirmation(self):
        self.given_service_interface_has_action(
            ServiceActionInterface("AlarmRings", self.mock_service_target, parameters=[], failure_reasons=[])
        )
        self.given_action_is_not_postconfirmed("AlarmRings")
        self.given_compulsory_grammar_for_app()
        self.given_grammar([
            Node(Constants.REPORT_ENDED, {"action": "AlarmRings"}, [Node(Constants.ITEM, {}, ["The alarm stopped"])])
        ])
        self.when_generating()
        self.expect_report_ended_was_generated_for_action("AlarmRings")

    def test_unexpected_parameter_name(self):
        self.given_service_interface_has_validity(
            ServiceValidatorInterface(
                "HourValidity", self.mock_service_target, parameters=[ServiceParameter("hour_to_set")]
            )
        )
        self.given_grammar([
            Node(
                Constants.VALIDITY, {"name": "HourValidity"}, [
                    Node(
                        Constants.ITEM, {},
                        ["cannot set the hour to ",
                         Node(Constants.SLOT, {"predicate": "dest_city"}), "."]
                    )
                ]
            )
        ])
        with self.assertRaises(UnexpectedParameter) as cm:
            self.when_generating()
        self.assertIn("unexpected parameter name dest_city ", unicode(cm.exception))

    def test_warn_about_missing_entry_for_action_py_grammar(self):
        self.given_py_grammar()
        self.given_domain_has_plan_for_goal("perform(settings)")
        self.given_compulsory_grammar_for_app(exclude=(Constants.ACTION, {"name": "settings"}))
        self.when_generating()
        self.expect_warning(
            'How do speakers talk about the action settings? Specify the utterance:\n\nsettings = "settings"\n\nAlternatively, you can specify several possible utterances in a list:\n\nsettings = [\n  "settings one way",\n  "settings another way",\n  "settings <answer:city>",\n]'
        )

    def test_warn_about_missing_entry_for_action_xml_grammar(self):
        self.given_xml_grammar()
        self.given_domain_has_plan_for_goal("perform(settings)")
        self.given_compulsory_grammar_for_app(exclude=(Constants.ACTION, {"name": "settings"}))
        self.when_generating()
        self.expect_warning(
            'How do speakers talk about the action settings? Specify the utterance:\n\n  <action name="settings">settings</action>\n\nAlternatively, you can specify several possible utterances in a list:\n\n  <action name="settings">\n    <one-of>\n      <item>settings one way</item>\n      <item>settings another way</item>\n      <item>settings <slot predicate="city" type="individual"/></item>\n    </one-of>\n  </action>'
        )

    def test_warn_about_missing_entry_for_goal_issue_py_grammar(self):
        self.given_py_grammar()
        self.given_domain_has_plan_for_goal("resolve(?X.price(X))")
        self.given_compulsory_grammar_for_app()
        self.given_grammar([
            Node(Constants.USER_QUESTION, {"predicate": "price"}, [Node(Constants.ITEM, {}, ['what is the price'])])
        ])
        self.when_generating()
        self.expect_warning(
            'How do the speakers talk about the issue price? The entry is used in questions such as "Do you want to know X?" or "I want to know X", where X is the grammar entry.\n\nExample:\n\nprice = "price"\n\nAlternatively, you can specify several possible utterances in a list:\n\nprice = [\n  "price one way",\n  "price another way",\n]'
        )

    def test_warn_about_missing_entry_for_goal_issue_xml_grammar(self):
        self.given_xml_grammar()
        self.given_domain_has_plan_for_goal("resolve(?X.price(X))")
        self.given_compulsory_grammar_for_app()
        self.given_grammar([
            Node(Constants.USER_QUESTION, {"predicate": "price"}, [Node(Constants.ITEM, {}, ['what is the price'])])
        ])
        self.when_generating()
        self.expect_warning(
            'How do the speakers talk about the issue price? The entry is used in questions such as "Do you want to know X?" or "I want to know X", where X is the grammar entry.\n\nExample:\n\n  <question speaker="all" predicate="price" type="wh_question">price</question>\n\nAlternatively, you can specify several possible utterances in a list:\n\n  <question speaker="all" predicate="price" type="wh_question">\n    <one-of>\n      <item>price one way</item>\n      <item>price another way</item>\n    </one-of>\n  </question>'
        )

    def test_warn_about_missing_entry_for_user_question(self):
        self.given_domain_has_plan_for_goal("resolve(?X.price(X))")
        self.given_compulsory_grammar_for_app()
        self.given_grammar([Node(Constants.PREDICATE, {"name": "price"}, [Node(Constants.ITEM, {}, ['price'])])])
        self.when_generating()
        self.expect_warning(
            'How does the user ask about price?\n\nExample:\n\nprice_user_question = [\n  "what is price",\n  "i want to know price"\n]'
        )

    def test_warn_about_missing_entry_for_system_question_py_grammar(self):
        self.given_py_grammar()
        self.given_some_plan_contains("findout(?X.dest_city(X))")
        self.given_compulsory_grammar_for_app()
        self.when_generating()
        self.expect_warning(
            'How does the system ask about dest_city?\n\nExample:\n\ndest_city_sys_question = "what is dest city"'
        )

    def test_warn_about_missing_entry_for_system_question_xml_grammar(self):
        self.given_xml_grammar()
        self.given_some_plan_contains("findout(?X.dest_city(X))")
        self.given_compulsory_grammar_for_app()
        self.when_generating()
        self.expect_warning(
            'How does the system ask about dest_city?\n\nExample:\n\n  <question speaker="system" predicate="dest_city" type="wh_question">what is dest city</question>'
        )

    def test_not_warn_about_system_question_due_to_explicit_entry(self):
        self.given_some_plan_contains("findout(?X.dest_city(X))")
        self.given_compulsory_grammar_for_app()
        self.given_grammar([
            Node(
                Constants.SYS_QUESTION, {"predicate": "dest_city"},
                [Node(Constants.ITEM, {}, ["where do you want to go"])]
            )
        ])
        self.when_generating()
        self.expect_no_warning()

    def test_not_warn_about_system_question_due_to_general_entry(self):
        self.given_some_plan_contains("findout(?X.dest_city(X))")
        self.given_compulsory_grammar_for_app()
        self.given_grammar([
            Node(Constants.PREDICATE, {"name": "dest_city"}, [Node(Constants.ITEM, {}, ["the destination"])])
        ])
        self.when_generating()
        self.expect_no_warning()

    def test_warn_about_missing_entry_for_individual_py_grammar(self):
        self.given_py_grammar()
        self.given_compulsory_grammar_for_app(exclude=(Constants.INDIVIDUAL, {"name": "new_york"}))
        self.when_generating()
        self.expect_warning(
            'How do speakers talk about the individual new_york? Specify the utterance:\n\nnew_york = "new york"\n\nAlternatively, you can specify several possible utterances in a list:\n\nnew_york = [\n  "new york one way",\n  "new york another way",\n]'
        )

    def test_warn_about_missing_entry_for_individual_xml_grammar(self):
        self.given_xml_grammar()
        self.given_compulsory_grammar_for_app(exclude=(Constants.INDIVIDUAL, {"name": "new_york"}))
        self.when_generating()
        self.expect_warning(
            'How do speakers talk about the individual new_york? Specify the utterance:\n\n  <individual name="new_york">new york</individual>\n\nAlternatively, you can specify several possible utterances in a list:\n\n  <individual name="new_york">\n    <one-of>\n      <item>new york one way</item>\n      <item>new york another way</item>\n    </one-of>\n  </individual>'
        )

    def test_warn_about_missing_validity_entry(self):
        self.given_service_interface_has_validity(
            ServiceValidatorInterface(
                "CityValidity",
                self.mock_service_target,
                parameters=[
                    ServiceParameter("dept_city", is_optional=True),
                    ServiceParameter("dest_city", is_optional=True),
                ]
            )
        )
        self.given_compulsory_grammar_for_app()
        self.when_generating()
        self.expect_warning(
            'How does the system report that the device validity CityValidity is unsatisfied? Specify the utterance:\n\nCityValidity = "invalid parameters <answer:dept_city> <answer:dest_city>"'
        )

    def test_background_with_string_answer_raises_exception(self):
        self.given_compulsory_grammar_for_app()
        self.given_grammar([
            Node(
                Constants.SYS_ANSWER, {"predicate": "location"}, [
                    Node(
                        Constants.ITEM, {}, [
                            Node(Constants.SLOT, {"predicate": "frequent_flyer_points"}),
                            " blablabla ",
                            Node(Constants.SLOT, {"predicate": "location"}),
                        ]
                    )
                ]
            )
        ])
        with self.assertRaises(InvalidSortOfBackgroundPredicateException) as e:
            self.when_generating()
        self.assertIn(
            "Background is not allowed for predicate 'location' of sort 'string'. Perhaps you can create a new sort for it?",
            e.exception
        )

    def test_background_with_integer_answer_raises_exception(self):
        self.given_compulsory_grammar_for_app()
        self.given_grammar([
            Node(
                Constants.SYS_ANSWER, {"predicate": "frequent_flyer_points"}, [
                    Node(
                        Constants.ITEM, {}, [
                            Node(Constants.SLOT, {"predicate": "frequent_flyer_points"}),
                            " blablabla ",
                            Node(Constants.SLOT, {"predicate": "location"}),
                        ]
                    )
                ]
            )
        ])
        with self.assertRaises(InvalidSortOfBackgroundPredicateException) as e:
            self.when_generating()
        self.assertIn(
            "Background is not allowed for predicate 'frequent_flyer_points' of sort 'integer'. Perhaps you can create a new sort for it?",
            e.exception
        )

    def test_can_be_asked_by_system_true(self):
        self.given_some_plan_contains("findout(?X.dest_city(X))")
        self.given_generator()
        self.assertEquals(True, self.generator._can_be_asked_by_system("dest_city"))

    def test_can_be_asked_by_system_false(self):
        self.given_some_plan_contains("findout(?X.goal(X))")
        self.given_generator()
        self.assertEquals(False, self.generator._can_be_asked_by_system("dest_city"))

    def test_can_be_asked_by_system_false_for_features(self):
        self.given_some_plan_contains("findout(?X.dest_city(X))")
        self.given_generator()
        self.assertEquals(False, self.generator._can_be_asked_by_system("dest_city_type"))

    def given_device_has_mockup_actions(self, action_names):
        for action_name in action_names:
            setattr(self._device_class, action_name, self._create_service_action(action_name))

    def given_action_is_postconfirmed(self, action_name):
        self._add_to_any_plan("invoke_service_action(%s, {postconfirm=True})" % action_name)

    def given_action_is_not_postconfirmed(self, action_name):
        pass

    def given_action_is_preconfirmed(self, action_name):
        self._add_to_any_plan("invoke_service_action(%s, {preconfirm=interrogative})" % action_name)

    def given_action_is_prereported(self, action_name):
        self._add_to_any_plan("invoke_service_action(%s, {preconfirm=assertive})" % action_name)

    def given_domain_has_plan_for_goal(self, goal_as_string):
        self._ensure_ontology_exists()
        self._ensure_domain_exists()
        goal = self._parser.parse(goal_as_string)
        self._domain.plans[goal] = {"goal": goal, "plan": Plan([])}

    def given_some_plan_contains(self, plan_item_as_string):
        self._add_to_any_plan(plan_item_as_string)

    def _add_to_any_plan(self, plan_item_as_string):
        self._ensure_ontology_exists()
        self._ensure_domain_exists()
        plan_item = self._parser.parse(plan_item_as_string)
        any_goal = self._domain.plans.keys()[0]
        self._domain.plans[any_goal]["plan"].push(plan_item)

    def expect_warning(self, expected_warning):
        self.assertEquals(expected_warning, self.generator.warning.strip())

    def expect_no_warning(self):
        self.assertEquals("", self.generator.warning)

    def expect_report_ended_was_generated_for_action(self, action_name):
        self.assert_abstract_contains_function('report_ended_AlarmRings_1 : SysReportEnded;\n')


class SysAnswerTest(AutoGeneratorTestCase):
    def setUp(self):
        AutoGeneratorTestCase.setUp(self)
        self._form = None

    def test_default_unary_propositional_system_answer_of_custom_sort(self):
        self.given_generator()
        self.when_generating_system_answer_content("dest_city")
        self.assert_abstract("dest_city_sys_answer : Sort_city -> SysAnswer;\n")
        self.assert_semantic("dest_city_sys_answer individual = pp dest_city.s individual;\n")
        self.assert_natural_language("dest_city_sys_answer individual = answer (individual.s) individual.s;\n")

    def test_overridden_unary_propositional_system_answer_of_custom_sort(self):
        self.given_generator()
        self.given_grammar([
            Node(
                Constants.SYS_ANSWER, {"predicate": "dest_city"},
                [Node(Constants.ITEM, {}, [u"på ", Node(Constants.SLOT, {})])]
            )
        ])
        self.when_generating_system_answer_content("dest_city")
        self.assert_abstract("dest_city_sys_answer : Sort_city -> SysAnswer;\n")
        self.assert_semantic("dest_city_sys_answer individual = pp dest_city.s individual;\n")
        self.assert_natural_language(u'dest_city_sys_answer individual = answer ("på" ++ individual.s) individual.s;\n')

    def test_nullary_propositional_system_answer(self):
        self.given_generator()
        self.given_grammar([
            Node(
                Constants.POSITIVE_SYS_ANSWER, {"predicate": "qualified_for_membership"},
                [Node(Constants.ITEM, {}, ["you are qualified for membership"])]
            ),
            Node(
                Constants.NEGATIVE_SYS_ANSWER, {"predicate": "qualified_for_membership"},
                [Node(Constants.ITEM, {}, ["you are not qualified for membership"])]
            )
        ])
        self.when_generating_system_answer_content("qualified_for_membership")
        self.assert_abstract(
            "qualified_for_membership_positive_sys_answer : SysAnswer;\n"
            "qualified_for_membership_negative_sys_answer : SysAnswer;\n"
        )
        self.assert_semantic(
            'qualified_for_membership_positive_sys_answer = pp qualified_for_membership.s;\n'
            'qualified_for_membership_negative_sys_answer = pp ("~" ++ qualified_for_membership.s);\n'
        )
        self.assert_natural_language(
            'qualified_for_membership_positive_sys_answer = answer "you are qualified for membership";\n'
            'qualified_for_membership_negative_sys_answer = answer "you are not qualified for membership";\n'
        )

    def test_no_entry_for_nullary_predicate(self):
        self.given_generator()
        self.when_generating_system_answer_content("qualified_for_membership")
        self.assert_abstract("")
        self.assert_semantic("")
        self.assert_natural_language("")

    def test_nullary_no_sortal_user_answer(self):
        self.given_generator()
        self.given_grammar([
            Node(
                Constants.POSITIVE_SYS_ANSWER, {"predicate": "qualified_for_membership"},
                [Node(Constants.ITEM, {}, ["you are qualified for membership"])]
            ),
            Node(
                Constants.NEGATIVE_SYS_ANSWER, {"predicate": "qualified_for_membership"},
                [Node(Constants.ITEM, {}, ["you are not qualified for membership"])]
            )
        ])
        self.when_generating_sortal_answer_content("qualified_for_membership")
        self.assert_abstract("")
        self.assert_semantic("")
        self.assert_natural_language("")

    def test_default_integer_system_answer(self):
        self.given_generator()
        self.when_generating_system_answer_content("distance")
        self.assert_abstract("distance_sys_answer : Integer -> SysAnswer;\n")
        self.assert_semantic('distance_sys_answer individual = pp "distance" individual;\n')
        self.assert_natural_language("distance_sys_answer individual = individual;\n")

    def test_overridden_integer_system_answer(self):
        self.given_grammar([
            Node(
                Constants.SYS_ANSWER, {"predicate": "distance"},
                [Node(Constants.ITEM, {}, ["the distance is ", Node(Constants.SLOT, {}), "meters"])]
            )
        ])
        self.given_generator()
        self.when_generating_system_answer_content("distance")
        self.assert_abstract("distance_sys_answer : Integer -> SysAnswer;\n")
        self.assert_semantic('distance_sys_answer individual = pp "distance" individual;\n')
        self.assert_natural_language(
            'distance_sys_answer individual = answer ("the distance is" ++ individual.s ++ "meters") individual.s;\n'
        )

    def when_generating_system_answer_content(self, predicate_name):
        self.generator._grammar = self._grammar
        self.generator._generate_system_answer_content(self.get_predicate(predicate_name))

    def when_generating_sortal_answer_content(self, predicate_name):
        self.generator._grammar = self._grammar
        self.generator._generate_sortal_user_answer(
            self.get_predicate(predicate_name),
            self.get_predicate(predicate_name).getSort()
        )

    def test_default_string_system_answer(self):
        self.given_generator()
        self.when_generating_system_answer_content("location")
        self.assert_abstract_contains("location_sys_answer_0 : SysAnswer;\n")
        self.assert_semantic_contains('location_sys_answer_0 = pp "location" string_placeholder_0;\n')
        self.assert_natural_language_contains('location_sys_answer_0 = answer ("_STR0_");\n')

    def test_overridden_string_system_answer(self):
        self.given_grammar([
            Node(
                Constants.SYS_ANSWER, {"predicate": "location"},
                [Node(Constants.ITEM, {}, [u"du är på ", Node(Constants.SLOT, {})])]
            )
        ])
        self.given_generator()
        self.when_generating_system_answer_content("location")
        self.assert_abstract_contains("location_sys_answer_0 : SysAnswer;\n")
        self.assert_semantic_contains('location_sys_answer_0 = pp "location" string_placeholder_0;\n')
        self.assert_natural_language_contains(u'location_sys_answer_0 = answer ("du är på _STR0_");\n')

    def test_default_datetime_system_answer(self):
        self.given_generator()
        self.when_generating_system_answer_content("flight_departure")
        self.assert_abstract_contains("flight_departure_sys_answer_0 : SysAnswer;\n")
        self.assert_semantic_contains('flight_departure_sys_answer_0 = pp "flight_departure" datetime_placeholder_0;\n')
        self.assert_natural_language_contains('flight_departure_sys_answer_0 = answer ("_datetime_placeholder_0_");\n')

    def test_overridden_datetime_system_answer(self):
        self.given_grammar([
            Node(
                Constants.SYS_ANSWER, {"predicate": "flight_departure"},
                [Node(Constants.ITEM, {}, [u"The flight departs at", Node(Constants.SLOT, {})])]
            )
        ])
        self.given_generator()
        self.when_generating_system_answer_content("flight_departure")
        self.assert_abstract_contains("flight_departure_sys_answer_0 : SysAnswer;\n")
        self.assert_semantic_contains('flight_departure_sys_answer_0 = pp "flight_departure" datetime_placeholder_0;\n')
        self.assert_natural_language_contains(
            u'flight_departure_sys_answer_0 = answer ("The flight departs at" ++ "_datetime_placeholder_0_");\n'
        )


class UserAnswerTest(AutoGeneratorTestCase):
    def test_default_short_answer_content(self):
        self.given_grammar([])
        self.when_generating()
        self.assert_abstract_contains_function("city_user_answer : Sort_city -> UsrAnswer;\n")
        self.assert_semantic_contains_linearization("city_user_answer answer = answer;\n")
        self.assert_natural_language_contains_linearization("city_user_answer answer = answer;\n")

    def test_no_boolean_answer_content(self):
        self.given_grammar([])
        self.when_generating()
        self.assert_abstract_not_contains_function("boolean_user_answer : Sort_boolean -> UsrAnswer;\n")
        self.assert_semantic_not_contains_linearization("boolean_user_answer answer = answer;\n")
        self.assert_natural_language_not_contains_linearization("boolean_user_answer answer = answer;\n")

    def test_answer_combination(self):
        self.given_grammar([
            Node(
                Constants.ANSWER_COMBINATION, {}, [
                    Node(
                        Constants.ITEM, {}, [
                            Node(Constants.SLOT, {"predicate": "passenger_quantity_to_add"}), " ",
                            Node(Constants.SLOT, {"predicate": "passenger_type_to_add"})
                        ]
                    )
                ]
            )
        ])
        self.when_generating()
        self.assert_abstract_contains_function(
            "user_answer_sequence_1 : Predicate_passenger_quantity_to_add -> Predicate_passenger_type_to_add -> UserAnswerSequence;\n"
        )
        self.assert_semantic_contains_linearization(
            'user_answer_sequence_1 passenger_quantity_to_add passenger_type_to_add = moveseq (move "answer" passenger_quantity_to_add) (move "answer" passenger_type_to_add);\n'
        )
        self.assert_natural_language_contains_linearization(
            'user_answer_sequence_1 passenger_quantity_to_add passenger_type_to_add = ss (passenger_quantity_to_add.s ++ " " ++ passenger_type_to_add.s);\n'
        )

    def test_propositional_user_answer_for_custom_sort(self):
        self.given_grammar([])
        self.when_generating()
        self.assert_abstract_contains_function(
            "dest_city_propositional_usr_answer : Sort_city -> Predicate_dest_city;\n"
        )
        self.assert_semantic_contains_linearization(
            "dest_city_propositional_usr_answer answer = pp dest_city.s answer;\n"
        )
        self.assert_natural_language_contains_linearization('dest_city_propositional_usr_answer answer = answer;\n')

    def test_propositional_user_answer_for_string_sort(self):
        self.given_grammar([])
        self.when_generating()
        self.assert_abstract_contains_function('comment_propositional_usr_answer : Unknown -> Predicate_comment;\n')
        self.assert_semantic_contains_linearization(
            'comment_propositional_usr_answer answer = pp comment.s (ss ("\\"" ++ answer.s ++ "\\""));\n'
        )
        self.assert_natural_language_contains_linearization('comment_propositional_usr_answer answer = answer;\n')

    def test_propositional_user_answer_for_integer_sort(self):
        self.given_grammar([])
        self.when_generating()
        self.assert_abstract_contains_function(
            'hour_to_set_propositional_usr_answer : Integer -> Predicate_hour_to_set;\n'
        )
        self.assert_semantic_contains_linearization(
            'hour_to_set_propositional_usr_answer answer = pp hour_to_set.s answer;\n'
        )
        self.assert_natural_language_contains_linearization('hour_to_set_propositional_usr_answer answer = answer;\n')

    def test_generate_sortal_user_answer(self):
        self.given_grammar([])
        self.when_generating()
        self.assert_abstract_contains_function("dest_city_sortal_usr_answer : Sort_city -> UsrAnswer;\n")
        self.assert_semantic_contains_linearization("dest_city_sortal_usr_answer answer = answer;\n")
        self.assert_natural_language_contains_linearization('dest_city_sortal_usr_answer answer = answer;\n')

    def test_generate_sortal_user_answer_with_sort_string(self):
        self.given_grammar([])
        self.when_generating()
        self.assert_generated_gf_contains_sort_user_answer_with_sort_string()

    def assert_generated_gf_contains_sort_user_answer_with_sort_string(self):
        self.assert_abstract_contains_function("comment_sortal_usr_answer : Sort_string -> UsrAnswer;\n")
        self.assert_semantic_contains_linearization("comment_sortal_usr_answer answer = answer;\n")
        self.assert_natural_language_contains_linearization('comment_sortal_usr_answer answer = answer;\n')

    def test_generate_unknown_category(self):
        self.given_grammar([])
        self.when_generating()
        self.assert_unknown_category_is_generated()

    def assert_unknown_category_is_generated(self):
        self.assert_abstract_contains_category(UNKNOWN_CATEGORY)
        self.assert_abstract_contains_function(UNKNOWN_FUNCTION)
        self.assert_abstract_contains_function(MK_UNKNOWN_FUNCTION)

        self.assert_semantic_contains_lincat(UNKNOWN_LINCAT)
        self.assert_semantic_contains_linearization(UNKNOWN_LINERARIZATION)
        self.assert_semantic_contains_linearization(MK_UNKNOWN_LINEARIZATION)

        self.assert_natural_language_contains_linearization(UNKNOWN_NL_LINEARIZATION)
        self.assert_natural_language_contains_linearization(MK_UNKNOWN_NL_LINEARIZATION)


class UserQuestionTest(AutoGeneratorTestCase):
    def test_without_answers(self):
        self.given_grammar([
            Node(Constants.USER_QUESTION, {"predicate": "price"}, [Node(Constants.ITEM, {}, ["price information"])])
        ])
        self.when_generating()
        self.assert_abstract_contains_function("ask_price : UsrWHQ;\n")
        self.assert_semantic_contains_linearization("ask_price = ask_whq price;\n")
        self.assert_natural_language_contains_linearization('ask_price = ss (("price information"));\n')

    def test_with_sortal_answer_of_custom_sort(self):
        self.given_grammar([
            Node(
                Constants.USER_QUESTION, {"predicate": "price"},
                [Node(Constants.ITEM, {}, ["price to ", Node(Constants.SLOT, {"sort": "city"})])]
            )
        ])
        self.when_generating()
        self.assert_abstract_contains_function('price_user_question_1 : Sort_city -> UsrWHQ;\n')
        self.assert_semantic_contains_linearization("price_user_question_1 city = ask_whq price city;\n")
        self.assert_natural_language_contains_linearization(
            'price_user_question_1 city = ss ("price to " ++ city.s);\n'
        )

    def test_with_propositional_answer_of_custom_sort(self):
        self.given_grammar([
            Node(
                Constants.USER_QUESTION, {"predicate": "price"},
                [Node(Constants.ITEM, {}, ["price to ", Node(Constants.SLOT, {"predicate": "dest_city"})])]
            )
        ])
        self.when_generating()
        self.assert_abstract_contains_function("price_user_question_1 : Predicate_dest_city -> UsrWHQ;\n")
        self.assert_semantic_contains_linearization("price_user_question_1 dest_city = ask_whq price dest_city;\n")
        self.assert_natural_language_contains_linearization(
            'price_user_question_1 dest_city = ss ("price to " ++ dest_city.s);\n'
        )

    def test_with_propositional_literal_answer(self):
        self.given_grammar([
            Node(
                Constants.USER_QUESTION, {"predicate": "attraction_information"},
                [Node(Constants.ITEM, {},
                      ["tell me about ", Node(Constants.SLOT, {"predicate": "attraction"})])]
            )
        ])
        self.given_generator()
        self.when_generating()
        self.assert_abstract_contains_function(
            'attraction_information_user_question_1 : Predicate_attraction -> UsrWHQ;\n'
        )
        self.assert_semantic_contains_linearization(
            'attraction_information_user_question_1 attraction = ask_whq attraction_information attraction;\n'
        )
        self.assert_natural_language_contains_linearization(
            'attraction_information_user_question_1 attraction = ss ("tell me about " ++ attraction.s);\n'
        )


class SysQuestionTest(AutoGeneratorTestCase):
    def test_base_case(self):
        self.given_form(Node(Constants.ITEM, {}, ["where do you want to go"]))
        self.given_generator()
        self.when_generating_system_question("dest_city")
        self.assert_natural_language('dest_city = ss "where do you want to go";\n')

    def when_generating_system_question(self, predicate_name):
        self.generator._generate_system_question(self.get_predicate(predicate_name), self._form)


class BackgroundEmbeddingTest(AutoGeneratorTestCase):
    def test_single_background_for_wh_question(self):
        self.given_form(Node(Constants.ITEM, {}, ["which hour ", Node(Constants.SLOT, {"predicate": "day_to_set"})]))
        self.given_generator()
        self.when_generating_predicate_content("hour_to_set")
        self.assert_abstract_contains('hour_to_set_ask_whq_with_background_1 : SysAnswer -> System;\n')
        self.assert_semantic_contains(
            'hour_to_set_ask_whq_with_background_1 day_to_set = pp "Move" (move "ask" (eta_expand hour_to_set) (list day_to_set));\n'
        )
        self.assert_natural_language_contains(
            'hour_to_set_ask_whq_with_background_1 day_to_set = ss (CAPIT ++ "which hour " ++ day_to_set.s ++ "?");\n'
        )

    def test_single_background_for_positive_nullary_answer(self):
        self.given_grammar([
            Node(
                Constants.POSITIVE_SYS_ANSWER, {"predicate": "qualified_for_membership"}, [
                    Node(
                        Constants.ITEM, {}, [
                            "you have ",
                            Node(Constants.SLOT, {"predicate": "frequent_flyer_points"}),
                            " frequent flyer points and are qualified for membership"
                        ]
                    )
                ]
            )
        ])
        self.given_generator()
        self.when_generating()
        self.assert_abstract_contains_function(
            'qualified_for_membership_positive_sys_answer : '
            'SysAnswer -> System;\n'
        )
        self.assert_semantic_contains_linearization(
            'qualified_for_membership_positive_sys_answer '
            'frequent_flyer_points = '
            'pp "Move" (move "answer" '
            '(qualified_for_membership) (list frequent_flyer_points));\n'
        )
        self.assert_natural_language_contains_linearization(
            'qualified_for_membership_positive_sys_answer '
            'frequent_flyer_points = '
            'ss (CAPIT ++ "you have" ++ frequent_flyer_points.s ++ '
            '"frequent flyer points and are qualified for membership" '
            '++ ".");\n'
        )

    def test_multi_background_for_positive_nullary_answer(self):
        self.given_grammar([
            Node(
                Constants.POSITIVE_SYS_ANSWER, {"predicate": "qualified_for_membership"}, [
                    Node(
                        Constants.ITEM, {}, [
                            "you have ",
                            Node(Constants.SLOT, {"predicate": "frequent_flyer_points"}),
                            " frequent flyer points and are qualified for "
                            "membership and you are in ",
                            Node(Constants.SLOT, {"predicate": "current_position"})
                        ]
                    )
                ]
            )
        ])
        self.given_generator()
        self.when_generating()
        self.assert_abstract_contains_function(
            'qualified_for_membership_positive_sys_answer : '
            'SysAnswer -> SysAnswer -> System;\n'
        )
        self.assert_semantic_contains_linearization(
            'qualified_for_membership_positive_sys_answer '
            'frequent_flyer_points current_position = '
            'pp "Move" (move "answer" '
            '(qualified_for_membership) '
            '(list frequent_flyer_points current_position));\n'
        )
        self.assert_natural_language_contains_linearization(
            'qualified_for_membership_positive_sys_answer '
            'frequent_flyer_points current_position = '
            'ss (CAPIT ++ "you have" ++ frequent_flyer_points.s ++ '
            '"frequent flyer points and are qualified for membership '
            'and you are in" ++ current_position.s ++ ".");\n'
        )

    def test_background_for_propositional_answer_of_custom_sort(self):
        self.given_grammar([
            Node(
                Constants.SYS_ANSWER, {"predicate": "next_membership_level"}, [
                    Node(
                        Constants.ITEM, {}, [
                            "you need ",
                            Node(Constants.SLOT, {"predicate": "next_membership_points"}), " to reach ",
                            Node(Constants.SLOT, {}), " level"
                        ]
                    )
                ]
            )
        ])
        self.given_generator()
        self.when_generating()
        self.assert_abstract_contains_function(
            'next_membership_level_sys_answer : '
            'Sort_membership -> SysAnswer -> System;\n'
        )
        self.assert_semantic_contains_linearization(
            'next_membership_level_sys_answer '
            'individual next_membership_points = '
            'pp "Move" (move "answer" '
            '(pp next_membership_level.s individual) (list next_membership_points));\n'
        )
        self.assert_natural_language_contains_linearization(
            'next_membership_level_sys_answer '
            'individual next_membership_points = '
            'ss (CAPIT ++ "you need" ++ next_membership_points.s ++ '
            '"to reach" ++ individual.s ++ "level" ++ ".");\n'
        )


class SysResolveYnqTest(AutoGeneratorTestCase):
    def test_base_case_no_background(self):
        self.given_form(Node(Constants.ITEM, {}, ["the price"]))
        self.given_generator()
        self.when_generating_predicate_content("price")
        self.assert_abstract_contains('price_resolve_ynq_1 : SysResolveGoal;\n')
        self.assert_semantic_contains('price_resolve_ynq_1 = resolve_ynq price;\n')
        self.assert_natural_language_contains('price_resolve_ynq_1 = resolve_ynq price;\n')

    def test_single_background(self):
        self.given_form(Node(Constants.ITEM, {}, ["the price ", Node(Constants.SLOT, {"predicate": "dest_city"})]))
        self.given_generator()
        self.when_generating_predicate_content("price")
        self.assert_abstract_contains('price_resolve_ynq_2 : SysAnswer -> SysResolveGoal;\n')
        self.assert_semantic_contains(
            'price_resolve_ynq_2 dest_city = resolve_ynq_with_background price (list dest_city);\n'
        )
        self.assert_natural_language_contains(
            'price_resolve_ynq_2 dest_city = resolve_ynq_with_background (ss ("the price " ++ dest_city.s));\n'
        )


class DevicelessDddTest(AutoGeneratorTestCase):
    def test_deviceless_ddd(self):
        self._given_generator_created_for_deviceless_ddd()
        self._when_generating()
        self._then_no_exception_is_raised()

    def _given_generator_created_for_deviceless_ddd(self):
        self._create_mock_ddd()

        def get_grammar(language_code):
            return self._grammar

        self.generator = MockAutoGenerator(self._ddd)
        self.generator._load_and_compile_grammar_entries = get_grammar
        self.generator.warning = ""
        self.generator._grammar_compiler = self._grammar_compiler

    def _when_generating(self):
        self.generator.generate("eng")


class GreetingTest(AutoGeneratorTestCase):
    def test_greeting(self):
        self.given_grammar([Node(Constants.GREETING, {}, [Node(Constants.ITEM, {}, ["Welcome to the Service!"])])])
        self.given_generator()
        self.when_generating()
        self.assert_natural_language_contains_linearization('sysGreet = ss "Welcome to the Service!";')

    def test_greeting_entry_is_optional(self):
        self.given_grammar([])
        self.given_generator()
        self.when_generating()
        self._then_no_exception_is_raised()

    def test_greeting_contains_unicode(self):
        self.given_grammar([Node(Constants.GREETING, {}, [Node(Constants.ITEM, {}, [u"Hallå!"])])])
        self.given_generator()
        self.when_generating()
        self.assert_natural_language_contains_linearization(u'sysGreet = ss "Hallå!";')


class IndividualTest(AutoGeneratorTestCase):
    def test_base_case(self):
        self.given_grammar([Node(Constants.INDIVIDUAL, {"name": "new_york"}, [Node(Constants.ITEM, {}, ["New York"])])])
        self.given_generator()
        self.when_generating()
        self.assert_abstract_contains_function('new_york : Sort_city;\n')
        self.assert_semantic_contains_linearization('new_york = pp "new_york";\n')
        self.assert_natural_language_contains_linearization('new_york = ss (("New York"));\n')

    def test_placeholders_for_dynamic_sorts(self):
        self.given_grammar([])
        self.given_generator()
        self.when_generating()
        self.assert_abstract_contains_function('placeholder_keyword0 : Sort_keyword;\n')
        self.assert_semantic_contains_linearization('placeholder_keyword0 = pp "_sem_placeholder_keyword0_";\n')
        self.assert_natural_language_contains_linearization('placeholder_keyword0 = ss "_nl_placeholder_keyword0_";\n')


class PlaceholdersForBuiltinSortsTestCase(AutoGeneratorTestCase):
    def test_integer(self):
        self.given_grammar([])
        self.given_generator()
        self.when_generating()
        self.assert_abstract_contains_function('integer_placeholder_0 : Integer;\n')
        self.assert_semantic_contains_linearization('integer_placeholder_0 = pp "_integer_placeholder_0_";\n')
        self.assert_natural_language_contains_linearization(
            'integer_placeholder_0 = integer "_integer_placeholder_0_" "_integer_placeholder_0_";\n'
        )

    def test_datetime(self):
        self.given_grammar([])
        self.given_generator()
        self.when_generating()
        self.assert_abstract_contains_function('datetime_placeholder_0 : DateTime;\n')
        self.assert_semantic_contains_linearization('datetime_placeholder_0 = pp "_datetime_placeholder_0_";\n')
        self.assert_natural_language_contains_linearization('datetime_placeholder_0 = ss "_datetime_placeholder_0_";\n')


class LoadingAndCompilationTestCase(AutoGeneratorTestCase):
    def setUp(self):
        self._temp_dir = tempfile.mkdtemp(prefix="AutoGeneratorTestCase")
        self._cwd = os.getcwd()
        self._schema_absolute_path = os.path.abspath(tala.ddd.schemas.__path__[0])
        os.chdir(self._temp_dir)
        AutoGeneratorTestCase.setUp(self)

    def tearDown(self):
        os.chdir(self._cwd)
        shutil.rmtree(self._temp_dir)

    def test_load_and_compile_grammar_entries_from_py(self):
        self._given_grammar_source_file("grammar_eng.py", 'top = "start view"')
        self._when_load_and_compile_grammar_entries("eng")
        self._then_result_is(
            Node(
                Constants.GRAMMAR, {},
                [Node(Constants.ACTION, {"name": "top"}, [Node(Constants.ITEM, {}, ["start view"])])]
            )
        )

    def _given_grammar_source_file(self, filename, contents):
        with open(filename, "w") as f:
            f.write(contents)

    def _when_load_and_compile_grammar_entries(self, language_code):
        self._create_mock_ddd()

        def get_grammar(language_code):
            return self._grammar

        generator = MockAutoGenerator(self._ddd)
        self._result = generator._load_and_compile_grammar_entries(language_code)

    def _then_result_is(self, expected_result):
        self.assertEquals(expected_result, self._result)

    def test_load_and_compile_grammar_entries_from_xml(self):
        self._given_grammar_source_file("grammar_eng.xml", '<grammar><action name="top">start view</action></grammar>')
        self._when_load_and_compile_grammar_entries("eng")
        self._then_result_is(
            Node(
                Constants.GRAMMAR, {},
                [Node(Constants.ACTION, {"name": "top"}, [Node(Constants.ITEM, {}, ["start view"])])]
            )
        )

    def test_validate_valid_xml_grammar(self):
        self._given_grammar_source_file("grammar_eng.xml", '<grammar><action name="top">start view</action></grammar>')
        self._when_load_and_compile_grammar_entries("eng")
        self._then_no_exception_is_raised()

    def test_validate_invalid_xml_grammar(self):
        with self.assertRaises(ViolatesSchemaException) as context:
            self._given_grammar_source_file(
                "grammar_eng.xml", '<grammar><invalid_element name="top">start view</invalid_element></grammar>'
            )
            self._when_load_and_compile_grammar_entries("eng")
        self.assertIn(
            "Expected grammar_eng.xml compliant with schema but it's in violation: "
            "Element 'invalid_element': This element is not expected., line 1", context.exception
        )



class CommonExample(object):
    def __init__(self, intent, text, entities=None):
        super(CommonExample, self).__init__()
        self._intent = intent
        self._text = text
        self._entities = entities or []

    @property
    def text(self):
        return self._text

    @property
    def intent(self):
        return self._intent

    @property
    def entities(self):
        return self._entities

    def generate(self):
        return {
            "text": self._text,
            "intent": self._intent,
            "entities": [entity.generate() for entity in self._entities],
        }



# coding: utf-8

from tala.nl.languages import ENGLISH, SWEDISH, SPANISH
from tala.nl.rasa.constants import NEGATIVE_INTENT
from tala.nl.rasa.generating.common_example import CommonExample


class SortNotSupportedException(Exception):
    pass


class Examples(object):
    @property
    def negative(self):
        raise NotImplementedError

    @property
    def integer(self):
        raise NotImplementedError

    @property
    def string(self):
        raise NotImplementedError

    @property
    def datetime(self):
        raise NotImplementedError

    def get_builtin_sort_examples(self, sort):
        if sort.is_domain_sort():
            return []
        if sort.is_integer_sort():
            return self.integer
        if sort.is_string_sort():
            return self.string
        if sort.is_datetime_sort():
            return self.datetime
        raise SortNotSupportedException("Builtin sort '%s' is not yet supported together with RASA" % sort.get_name())

    @staticmethod
    def from_language(language_code):
        examples = {
            ENGLISH: EnglishExamples(),
            SWEDISH: SwedishExamples(),
            SPANISH: SpanishExamples()
        }
        return examples[language_code]


class EnglishExamples(Examples):
    @property
    def negative(self):
        phrases = [
            "aboard", "about", "above", "across", "after", "against", "along", "among", "as", "at", "on", "atop",
            "before", "behind", "below", "beneath", "beside", "between", "beyond", "but", "by", "come", "down",
            "during", "except", "for", "from", "in", "inside", "into", "less", "like", "near", "of", "off", "on",
            "onto", "opposite", "out", "outside", "over", "past", "save", "short", "since", "than", "then", "through",
            "throughout", "to", "toward", "under", "underneath", "unlike", "until", "up", "upon", "with", "within",
            "without", "worth", "is", "it", "the", "a", "am", "are", "them", "this", "that", "I", "you", "he", "she",
            "they", "them", "his", "her", "my", "mine", "their", "your", "us", "our"
        ]
        question_phrases = [
            "how", "how's", "how is", "how's the", "how is the", "when", "when's", "when is", "when is the",
            "when's the", "what", "what is", "what's", "what's the", "what is the", "why", "why is", "why's",
            "why is the", "why's the"
        ]
        action_phrases = [
            "do", "make", "tell", "start", "stop", "enable", "disable", "raise", "lower", "decrease", "increase", "act",
            "determine", "say", "ask", "go", "shoot", "wait", "hang on", "ok", "show", "help"
        ]
        intent = NEGATIVE_INTENT
        for phrase in phrases:
            yield CommonExample(intent, phrase)
        for phrase in question_phrases:
            yield CommonExample(intent, phrase)
        for phrase in action_phrases:
            yield CommonExample(intent, phrase)

    @property
    def integer(self):
        return ["0", "99", "1224", "a hundred and fifty seven", "three", "two thousand fifteen"]

    @property
    def string(self):
        return [
            "single", "double word", "three in one", "hey make it four", "the more the merrier five",
            "calm down and count to six", "bring them through to the jolly seven",
            "noone counts toes like an eight toed guy", "it matters to make sense for nine of us",
            "would you bring ten or none to a desert island"
        ]

    @property
    def datetime(self):
        return [
            "today", "Monday March 18", "the 1st of March", "11:45 am", "next 3 weeks", "in ten minutes",
            "March 20th at 22:00", "March twentieth at 10 pm"
        ]


class SwedishExamples(Examples):
    @property
    def negative(self):
        phrases = [
            "om", u"ovanför", u"tvärsöver", "efter", "mot", "bland", "runt", "som", u"på", "vid", u"ovanpå", u"före",
            "bakom", "nedan", "under", "bredvid", "mellan", "bortom", "men", "av", "trots", "ner", u"förutom", u"för",
            u"från", "i", "inuti", "in i", u"nära", u"nästa", "mittemot", "ut", u"utanför", u"över", "per", "plus",
            "runt", "sedan", u"än", "genom", "tills", "till", "mot", "olik", "upp", "via", "med", "inom", "utan", u"är",
            "vara", "den", "det", "en", "ett", "dem", "denna", "detta", "jag", "du", "ni", "han", "hon", "hen", "de",
            "hans", "hennes", "hens", "min", "mina", "deras", "er", "din", "vi", "oss", u"vår"
        ]
        question_phrases = ["hur", u"hur är", u"när", u"när är", "vad", u"vad är", u"varför", u"varför är"]
        action_phrases = [
            u"gör", u"göra", "skapa", u"berätta", "tala om", u"börja", "starta", "sluta", "stopp", "stanna", u"sätt på",
            u"stäng av", u"höj", u"sänk", u"öka", "minska", "agera", u"bestäm", u"säg", u"fråga", u"gå", u"kör",
            u"vänta", "ok", "visa", u"hjälp"
        ]
        intent = NEGATIVE_INTENT
        for phrase in phrases:
            yield CommonExample(intent, phrase)
        for phrase in question_phrases:
            yield CommonExample(intent, phrase)
        for phrase in action_phrases:
            yield CommonExample(intent, phrase)

    @property
    def integer(self):
        return ["0", "99", "1224", "etthundratjugosju", "tre", u"tvåtusenfemton"]

    @property
    def string(self):
        return [
            "enkel", "dubbelt ord", "det blir tre", u"fyra på en gång", u"ju fler desto bättre fem",
            u"håll andan och räkna till sex", "led dem fram till de glada sju",
            u"ingen räknar tår som den med åtta tår", u"det spelar roll att det låter rimligt för nio",
            u"tar du med tio eller inga till en öde ö"
        ]

    @property
    def datetime(self):
        return [
            "idag", u"måndag 18 mars", "1:a mars", "klockan 11.45", u"följande tre veckor", "om tio minuter",
            u"20:e mars vid 22.00", u"tjugonde mars vid tio på kvällen"
        ]


class SpanishExamples(Examples):
    @property
    def negative(self):
        phrases = [
            "a bordo", "acerca de", "arriba", u"a través de", u"después de", "en contra", "a lo largo de", "entre",
            "como", "en", "en", "en lo alto", "antes", u"detrás", "abajo", "debajo", "al lado", "entre", u"más allá de",
            "pero", "por", "abajo", "durante", "excepto", "para", "desde", "en", "dentro", "en", "menos", "como",
            "cerca", "de", "encima de", "sobre", "opuesto", "fuera", "fuera de", "corto", "desde", "que", "entonces",
            "a lo largo de", "hasta", "hacia", "debajo de", "a diferencia de", "hasta", "arriba", "con", "dentro de",
            "sin", "vale", "es" "se", "el", "la" "a", "soy", "son", "ellos", "este", "ese", "yo", "usted ", u"él",
            "ella", "ellos", "ellas", "su", "sus", "mi", "tu", u"tú", "nosotros", "nosotras", "vosotros", "vosotras",
            "nuestro", "nuestra", "vuestro", "vuestra", "vuestros", "vuestras", u"mío", u"mía", u"míos", u"mías",
            "tuyo", "tuyos", "tuya", "tuyas", "suyo", "suya", "suyos", "suyas"
        ]
        question_phrases = [
            u"cómo", u"cómo está", u"cómo es", u"cómo está el", u"cómo es el", u"cómo está la", u"cómo es la",
            u"cómo están los", u"cómo están las"
            u"cuándo", u"cuándo es", u"cuándo está", u"cuándo es el", u"cuándo es la", u"cuándo son los",
            u"cuándo son las", u"cuándo está el", u"cuándo está la", u"cuándo están los",
            u"cuándo están las", u"qué", u"qué es", u"qué es la", u"qué es el", u"qué son los", u"qué son las",
            u"cuál", u"cuál es", u"cuál es la", u"cuál es el", u"cuáles son los", u"cuáles son las",
            u"por qué", u"por qué es", u"por qué está", u"por qué es el", u"por qué es la", u"por qué son",
            u"por qué son los", u"por qué son las", u"por qué está el", u"por qué está la",
            u"por qué están los", u"por qué están las"
        ]
        action_phrases = [
            "hacer", "decir", "iniciar", "detener", "habilitar", "deshabilitar", "querer", "dar", "haber"
            "subir", "bajar", "disminuir", "aumentar", "actuar", "determinar", "preguntar", "ir", "disparar",
            "esperar", "esperar", "aceptar", "mostrar", u"enseñar", "ayudar"
        ]
        intent = NEGATIVE_INTENT
        for phrase in phrases:
            yield CommonExample(intent, phrase)
        for phrase in question_phrases:
            yield CommonExample(intent, phrase)
        for phrase in action_phrases:
            yield CommonExample(intent, phrase)

    @property
    def integer(self):
        return ["0", "99", "1224", "100000", "100.000", "una", "uno", u"dieciséis", "veintiuno", "veintiuno",
                "veinte y uno", "tres", "dos mil quince", "mil cincuenta y siete"]

    @property
    def string(self):
        return [
            "singular", "doble palabra", "tres en uno", "hey pon cuatro", u"cuanto más mejor cinco",
            u"cálmate y cuenta hasta seis", u"llévalos hasta el siete",
            "nadie cuenta los dedos de los pies como un chico de ocho dedos",
            "importa tener sentido para nueve de nosotros", u"llevarías diez o ninguno a una isla desierta"
        ]

    @property
    def datetime(self):
        return [
            "hoy", "ayer", "este lunes", u"miércoles", "viernes 18 de febrero", "20 de febrero", "el 1 de marzo",
            "11:45 de la noche", "a las tres y quince", "la semana que viene", "en cinco minutos",
            u"próximos tres meses", "este fin de semana", u"el 12 de marzo a las 8 de la mañana"
        ]



import warnings

from tala.backend.dependencies.abstract_backend_dependencies import AbstractBackendDependencies
from tala.config import RasaConfig
from tala.ddd.loading.ddd_set_loader import DDDSetLoader
from tala.model.sort import DATETIME, INTEGER


class BackendDependenciesForGenerating(AbstractBackendDependencies):
    def __init__(self, backend_args):
        super(BackendDependenciesForGenerating, self).__init__(backend_args)
        self._raw_rasa_config = None
        self._is_duckling_enabled = False
        self._duckling_url = None
        if any(self.ddds_with_rasa):
            self._raw_rasa_config = RasaConfig(backend_args.rasa_config).read()
            self._is_duckling_enabled = self._raw_rasa_config["enable_duckling"]
            self._duckling_url = self._raw_rasa_config["duckling_url"]

        if any(self.ddds_with_rasa) and not self.is_duckling_enabled:
            self._validate_compatibility_with_rasa_but_not_duckling()

        self._validate_compatibility_without_rasa()

    def _validate_compatibility_with_rasa_but_not_duckling(self):
        for ddd in self.ddds_with_rasa:
            if ddd.ontology.predicates_contain_sort(DATETIME):
                warnings.warn(
                    "DDD '{0}' contains predicates of the '{1}' sort, but duckling is disabled. "
                    "'{1}' entities won't be recognized in NLU.".format(ddd.name, DATETIME)
                )
            if ddd.ontology.predicates_contain_sort(INTEGER):
                warnings.warn(
                    "DDD '{0}' contains predicates of the '{1}' sort, but duckling is disabled. "
                    "Accuracy for '{1}' entities will be reduced in NLU.".format(ddd.name, INTEGER)
                )

    def _validate_compatibility_without_rasa(self):
        for ddd in self.ddds_without_rasa:
            if ddd.ontology.predicates_contain_sort(DATETIME):
                warnings.warn(
                    "DDD '{0}' contains predicates of the '{1}' sort, but RASA NLU is disabled. "
                    "'{1}' entities won't be recognized in NLU.".format(ddd.name, DATETIME)
                )

    def load_ddds(self, ddd_names):
        ddd_set_loader = self._create_ddd_set_loader()
        ddds = ddd_set_loader.ddds_as_list(ddd_names, languages=self.supported_languages)
        return ddds

    def _create_ddd_set_loader(self):
        return DDDSetLoader(self.overridden_ddd_config_paths)

    @property
    def is_duckling_enabled(self):
        return self._is_duckling_enabled

    @property
    def duckling_url(self):
        return self._duckling_url

    @property
    def ddds_without_rasa(self):
        return [ddd for ddd in self.ddds if not ddd.is_rasa_enabled]



from tala.nl.rasa.generating.entity import PropositionalEntity, SortalEntity


class EntityFactory(object):
    def create(self, identifier, text, start, end):
        raise NotImplementedError("%s.create(...) need to be implemented" % self.__class__.__name__)


class PropositionalEntityFactory(EntityFactory):
    def create(self, identifier, text, start, end):
        return PropositionalEntity(identifier, text, start, end)


class SortalEntityFactory(EntityFactory):
    def create(self, identifier, text, start, end):
        return SortalEntity(identifier, text, start, end)



import os
import warnings

from tala.model.ddd import DDD
from tala.ddd.loading.python_module_loader import PythonModuleLoader, DddLoaderException
from tala.ddd.ddd_py_compiler import DddPyCompiler, DomainCompiler as DomainPyCompiler
from tala.ddd.ddd_xml_compiler import DddXmlCompiler, DomainCompiler as DomainXmlCompiler
from tala.ddd.parser import Parser
from tala.ddd.services.service_interface import ServiceInterface
from tala.ddd.services.service_interface_from_device import ServiceInterfaceFromDevice
from tala.model.grammar.grammar import Grammar, GrammarForRGL
from tala.ddd.grammar.reader import GrammarReader
from tala.ddd.grammar.parser import GrammarParser
from tala.model.domain import Domain, DddDomain
from tala.model.ontology import Ontology, DddOntology
from tala.utils import chdir


class DDDLoader(object):
    def __init__(self, name, ddd_config, languages):
        super(DDDLoader, self).__init__()
        self._name = name
        self._ddd_config = ddd_config
        self._languages = languages
        self._py_compiler = DddPyCompiler()
        self._xml_compiler = DddXmlCompiler()

        if ddd_config["enable_rasa_nlu"]:
            warnings.warn(
                "The support for RASA NLU is still in BETA. Talk gently. It is currently enabled in DDD '%s'." % name
            )

    def _load_grammar(self, language_code):
        if GrammarReader.xml_grammar_exists_for_language(language_code, path="grammar"):
            grammar_string = GrammarReader.read(language_code, path="grammar")
            grammar_path = GrammarReader.path(language_code, path="grammar")
            grammar_root = GrammarParser.parse(grammar_string)
            if self._ddd_config["use_rgl"]:
                return GrammarForRGL(grammar_root, grammar_path)
            else:
                return Grammar(grammar_root, grammar_path)
        return None

    def _compile_ontology(self):
        if os.path.exists("ontology.xml"):
            ontology_xml = self._load_xml_resource("ontology.xml")
            ontology_args = self._xml_compiler.compile_ontology(ontology_xml)
        elif os.path.exists("ontology.py"):
            warnings.warn("ontology.py is deprecated. Convert it to an ontology.xml instead.", DeprecationWarning)
            ontology_class = PythonModuleLoader(self._name).load_py_module_class("ontology.py", DddOntology)
            ontology_args = self._py_compiler.compile_ontology(ontology_class)
        else:
            raise DddLoaderException("neither .py nor .xml ontology found")

        return Ontology(**ontology_args)

    def _compile_service_interface(self):
        if not os.path.exists("service_interface.xml"):
            empty_service_interface = ServiceInterface([], [], [], [])
            suggestion = ServiceInterfaceFromDevice.to_xml(empty_service_interface)
            raise DddLoaderException(
                "Expected 'service_interface.xml' to exist but it does not. Start by adding an empty one:\n\n%s\n" %
                suggestion
            )
        service_interface_xml = self._load_xml_resource("service_interface.xml")
        return self._xml_compiler.compile_service_interface(service_interface_xml)

    def _compile_domain(self, ontology, parser, service_interface):
        domain_args = self._domain_as_dict(ontology, parser, service_interface)
        return Domain(ontology=ontology, **domain_args)

    def _domain_as_dict(self, ontology, parser, service_interface):
        if os.path.exists("domain.xml"):
            domain_xml = self._load_xml_resource("domain.xml")
            domain_as_dict = self._xml_compiler.compile_domain(
                self._name, domain_xml, ontology, parser, service_interface
            )
        elif os.path.exists("domain.py"):
            domain_class = PythonModuleLoader(self._name).load_py_module_class("domain.py", DddDomain)
            domain_as_dict = self._py_compiler.compile_domain(self._name, domain_class, ontology, parser)
        else:
            raise DddLoaderException("neither .py nor .xml domain found")
        return domain_as_dict

    def _load_xml_resource(self, resource_name):
        if os.path.exists(resource_name):
            with open(resource_name) as f:
                return f.read()
        else:
            raise DddLoaderException("Expected '%s' to exist but it does not" % resource_name)

    def _find_domain_name(self):
        if os.path.exists("domain.xml"):
            domain_xml = self._load_xml_resource("domain.xml")
            name = DomainXmlCompiler().get_name(domain_xml)
        elif os.path.exists("domain.py"):
            warnings.warn("domain.py is deprecated. Convert it to a domain.xml instead.", DeprecationWarning)
            domain_class = PythonModuleLoader(self._name).load_py_module_class("domain.py", DddDomain)
            name = DomainPyCompiler().get_name(domain_class)
        else:
            raise DddLoaderException("neither .py nor .xml domain found")
        return name

    def load(self):
        path = os.path.join(os.getcwd(), self._name)

        with chdir.chdir(self._name):
            ontology = self._compile_ontology()
            grammars = {}
            for language_code in self._languages:
                grammar = self._load_grammar(language_code)
                grammars[language_code] = grammar

            domain_name = self._find_domain_name()
            parser = Parser(self._name, ontology, domain_name)
            service_interface = self._compile_service_interface()
            domain = self._compile_domain(ontology, parser, service_interface)

        ddd = self._create_ddd(ontology, domain, service_interface, grammars)
        ddd.path = path
        return ddd

    def _create_ddd(self, ontology, domain, service_interface, grammars):
        return DDD(
            self._name, ontology, domain, self._ddd_config["enable_rasa_nlu"], service_interface, grammars,
            self._languages, self._ddd_config["use_rgl"]
        )



import json
import os
import datetime

from tala.utils.text_formatting import readable_list

DEFAULT_INACTIVE_SECONDS_ALLOWED = datetime.timedelta(hours=2).seconds


class ConfigNotFoundException(Exception):
    def __init__(self, message, config_path):
        self._config_path = config_path
        super(ConfigNotFoundException, self).__init__(message)

    @property
    def config_path(self):
        return self._config_path


class BackendConfigNotFoundException(ConfigNotFoundException):
    pass


class DddConfigNotFoundException(ConfigNotFoundException):
    pass


class RasaConfigNotFoundException(ConfigNotFoundException):
    pass


class UnexpectedConfigEntriesException(Exception):
    pass


class Config(object):
    def __init__(self, path=None):
        self._path = path
        if self._path is None:
            self._path = self.default_name()

    @property
    def _absolute_path(self):
        return os.path.join(os.getcwd(), self._path)

    def read(self):
        if not os.path.exists(self._path):
            self._handle_non_existing_config_file()
        self._potentially_update_and_backup_config()
        config = self._config_from_file()
        config = self._potentially_update_with_values_from_parent(config)
        return config

    def _handle_non_existing_config_file(self):
        self._raise_config_not_found_exception()

    def _write(self, config):
        self._write_to_file(config, self._path)

    @classmethod
    def write_default_config(cls, path=None):
        path = path or cls.default_name()
        cls._write_to_file(cls.default_config(), path)

    def _write_backup(self, config):
        self._write_to_file(config, self.back_up_name())

    @staticmethod
    def _write_to_file(config, path):
        with open(path, mode="w") as f:
            json.dump(config, f, sort_keys=True, indent=4, separators=(',', ': '))

    def _config_from_file(self):
        with open(self._path, mode="r") as f:
            return json.load(f)

    def _potentially_update_and_backup_config(self):
        config = self._config_from_file()
        config = self._potentially_update_with_values_from_parent(config)
        default_config = self.default_config()
        keys_different_from_default = set(default_config.keys()).symmetric_difference(config.keys())
        if keys_different_from_default:
            updated_config = self._update_config_keys(default_config, config)
            self._write_backup(config)
            self._write(updated_config)
            message = self._format_unexpected_entries_message(default_config.keys(), config.keys())
            raise UnexpectedConfigEntriesException(message)

    def _potentially_update_with_values_from_parent(self, config):
        parent_path = config.get("overrides")
        if parent_path:
            config = self._update_with_values_from_parent(config, parent_path)
        return config

    def _update_with_values_from_parent(self, config, parent_path):
        parent_config_object = self.__class__(parent_path)
        parent_config = parent_config_object.read()
        parent_config.update(config)
        return parent_config

    def _format_unexpected_entries_message(self, expected, actual):
        unexpected = list(set(actual).difference(expected))
        missing = list(set(expected).difference(actual))
        ending = "The config was updated and the previous config was backed up in %r." % self.back_up_name()
        if unexpected and missing:
            return "Parameter %s is unexpected while %s is missing from %r. %s" % (
                readable_list(unexpected), readable_list(missing), self._absolute_path, ending
            )
        if unexpected:
            return "Parameter %s is unexpected in %r. %s" % (readable_list(unexpected), self._absolute_path, ending)
        if missing:
            return "Parameter %s is missing from %r. %s" % (readable_list(missing), self._absolute_path, ending)

    @classmethod
    def _update_config_keys(cls, defaults, config):
        updated_config = {}
        for key, default_value in defaults.iteritems():
            if key in config:
                updated_config[key] = config[key]
            else:
                updated_config[key] = default_value
        return updated_config

    def back_up_name(self):
        return "%s.backup" % self._path

    @staticmethod
    def default_name():
        raise NotImplementedError()

    @staticmethod
    def default_config():
        raise NotImplementedError()

    def _raise_config_not_found_exception(self):
        raise NotImplementedError()


class BackendConfig(Config):
    @staticmethod
    def default_name():
        return "backend.config.json"

    @staticmethod
    def default_config(ddd_name=""):
        return {
            "supported_languages": ["eng"],
            "ddds": [ddd_name],
            "active_ddd": ddd_name,
            "asr": "none",
            "use_recognition_profile": False,
            "repeat_questions": True,
            "use_word_list_correction": False,
            "overrides": None,
            "rerank_amount": BackendConfig.default_rerank_amount(),
            "inactive_seconds_allowed": DEFAULT_INACTIVE_SECONDS_ALLOWED,
        }

    @staticmethod
    def default_rerank_amount():
        return 0.2

    @classmethod
    def _update_config_keys(cls, defaults, config):
        updated_config = super(BackendConfig, cls)._update_config_keys(defaults, config)
        if "use_word_list_correction" not in config and config.get("asr") == "android":
            updated_config["use_word_list_correction"] = True
        return updated_config

    @classmethod
    def write_default_config(cls, path=None, ddd_name=None):
        path = path or cls.default_name()
        ddd_name = ddd_name or ""
        cls._write_to_file(cls.default_config(ddd_name), path)

    def _raise_config_not_found_exception(self):
        raise BackendConfigNotFoundException(
            "Expected backend config '%s' to exist but it was not found." % self._absolute_path, self._absolute_path
        )


class DddConfig(Config):
    @staticmethod
    def default_name():
        return "ddd.config.json"

    @staticmethod
    def default_config():
        return {
            "use_rgl": True,
            "use_third_party_parser": False,
            "device_module": None,
            "word_list": "word_list.txt",
            "overrides": None,
            "enable_rasa_nlu": False,
        }

    def _raise_config_not_found_exception(self):
        raise DddConfigNotFoundException(
            "Expected DDD config '%s' to exist but it was not found." % self._absolute_path, self._absolute_path
        )


class RasaConfig(Config):
    @staticmethod
    def default_name():
        return "rasa.config.json"

    @staticmethod
    def default_config():
        return {
            "enable_duckling": False,
            "duckling_url": "http://127.0.0.1:10090",
        }

    def _raise_config_not_found_exception(self):
        raise RasaConfigNotFoundException(
            "Expected RASA config '%s' to exist but it was not found." % self._absolute_path, self._absolute_path
        )


class OverriddenDddConfig(object):
    def __init__(self, ddd_name, path):
        self._ddd_name = ddd_name
        self._path = path

    @property
    def ddd_name(self):
        return self._ddd_name

    @property
    def path(self):
        return self._path



import os

from tala.utils import chdir
from tala.nl.gf.naming import abstract_gf_filename, natural_language_gf_filename, semantic_gf_filename


def has_handcrafted_gf_grammar(ddd_name, language_code, path_to_grammar_folder):
    with chdir.chdir(path_to_grammar_folder):
        exists = (
            os.path.exists(abstract_gf_filename(ddd_name)) or os.path.exists(semantic_gf_filename(ddd_name))
            or os.path.exists(natural_language_gf_filename(ddd_name, language_code))
        )
        return exists


def path_to_grammar_build_folder(ddd_name, language_code, path):
    grammar_path = os.path.join(path, ddd_name, "grammar")
    is_handcrafted = has_handcrafted_gf_grammar(ddd_name, language_code, grammar_path)
    build_folder = "build_handcrafted" if is_handcrafted else "build"
    return os.path.join(grammar_path, build_folder)


class CacheMethod:
    def __init__(self, instance, method):
        self._method = method
        self._cache = {}

        def cached_method(*args):
            if args in self._cache:
                return self._cache[args]
            else:
                value = self._method.__call__(*args)
                self._cache[args] = value
                return value

        setattr(instance, method.__name__, cached_method)

    def __str__(self):
        return "CacheMethod(%s, _cache=%s)" % (self._method, self._cache)

    def clear(self):
        self._cache = {}



class NP:
    def __init__(self, indefinite, definite=None, gender=None, number=None):
        self.indefinite = indefinite
        self.definite = definite
        self.gender = gender
        self.number = number


class VP:
    def __init__(self, infinitive, imperative, ing_form, object_):
        self.infinitive = infinitive
        self.imperative = imperative
        self.ing_form = ing_form
        self.object = object_


class FEMININE:
    pass


class MASCULINE:
    pass


class PLURAL:
    pass


class SINGULAR:
    pass



from tala.model.speaker import Speaker
from tala.model.semantic_object import OntologySpecificSemanticObject, SemanticObject, SemanticObjectWithContent
from tala.utils.unicodify import unicodify


class Goal(SemanticObject):
    PERFORM_GOAL = "PERFORM_GOAL"
    RESOLVE_GOAL = "RESOLVE_GOAL"
    HANDLE_GOAL = "HANDLE_GOAL"

    def __init__(self, goal_type, target):
        self._goal_type = goal_type
        self._target = target
        self._background = None

    def is_goal(self):
        return True

    def is_perform_goal(self):
        return False

    def is_resolve_goal(self):
        return False

    def is_handle_goal(self):
        return False

    def get_goal_type(self):
        return self._goal_type

    def get_target(self):
        return self._target

    def __ne__(self, other):
        return not (self == other)

    def __eq__(self, other):
        try:
            return (other.get_target() == self.get_target() and other.get_goal_type() == self.get_goal_type())
        except AttributeError:
            return False

    def __hash__(self):
        return hash((self._goal_type, self._target))

    def __repr__(self):
        return "%s(%s, %s)" % (Goal.__name__, self._goal_type, self._target)

    def __unicode__(self):
        return repr(self)

    def set_background(self, background):
        self._background = background

    @staticmethod
    def goal_filter(goal_type):
        return lambda goal: goal.get_goal_type() == goal_type

    @staticmethod
    def goal_target_filter(target_speaker):
        return lambda goal: goal.get_target() == target_speaker


class GoalWithSemanticContent(Goal, SemanticObjectWithContent):
    def __init__(self, goal_type, target, content):
        Goal.__init__(self, goal_type, target)
        SemanticObjectWithContent.__init__(self, content)
        self._content = content

    def get_content(self):
        return self._content

    def is_goal_with_semantic_content(self):
        return True

    def __hash__(self):
        return hash((self._goal_type, self._target, self._content))

    def __repr__(self):
        return "%s(%s, %s, %s, %s)" % (
            GoalWithSemanticContent.__name__, self.ontology_name, self._goal_type, self._target, self._content
        )

    def __eq__(self, other):
        try:
            return (
                other.is_goal() and other.has_semantic_content() and other.get_content() == self.get_content()
                and other.get_target() == self.get_target() and other.get_goal_type() == self.get_goal_type()
            )
        except AttributeError:
            return False


class PerformGoal(GoalWithSemanticContent):
    def __init__(self, action):
        assert action.is_action()
        GoalWithSemanticContent.__init__(self, self.PERFORM_GOAL, Speaker.SYS, action)

    def is_perform_goal(self):
        return True

    @staticmethod
    def filter():
        return Goal.goal_filter(PerformGoal.PERFORM_GOAL)

    def get_action(self):
        return self.get_content()

    def __ne__(self, other):
        return not (self == other)

    def __unicode__(self):
        return "perform(%s)" % self.get_action()


class ResolveGoal(GoalWithSemanticContent):
    def __init__(self, question, target):
        assert question.is_question()
        GoalWithSemanticContent.__init__(self, self.RESOLVE_GOAL, target, question)

    def is_resolve_goal(self):
        return True

    def get_question(self):
        return self.get_content()

    @staticmethod
    def filter():
        return Goal.goal_filter(ResolveGoal.RESOLVE_GOAL)

    def __ne__(self, other):
        return not (self == other)

    def __unicode__(self):
        result = ""
        if self._target == Speaker.USR:
            result += "resolve_user"
        else:
            result += "resolve"
        result += "(%s" % self.get_question()
        if self._background:
            result += ", %s" % unicodify(self._background)
        result += ")"
        return result


class HandleGoal(Goal, OntologySpecificSemanticObject):
    def __init__(self, ontology_name, device_event):
        Goal.__init__(self, self.HANDLE_GOAL, Speaker.SYS)
        OntologySpecificSemanticObject.__init__(self, ontology_name)
        self._device_event = device_event

    def get_device_event(self):
        return self._device_event

    def __eq__(self, other):
        try:
            return (
                other.is_ontology_specific() and other.ontology_name == self.ontology_name and Goal.__eq__(self, other)
                and other.get_device_event() == self.get_device_event()
            )
        except AttributeError:
            return False

    def __ne__(self, other):
        return not (self == other)

    def is_handle_goal(self):
        return True

    def __unicode__(self):
        return "handle(%s)" % self.get_device_event()



import unittest
import warnings

from mock import patch, call

from tala.backend import dependencies
from tala.backend.dependencies.for_generating import BackendDependenciesForGenerating
from tala.backend.dependencies.test.base import BackendDependenciesBase
from tala.model.sort import DATETIME, INTEGER


class BackendDependenciesForGeneratingTestCase(BackendDependenciesBase, unittest.TestCase):
    def setUp(self):
        super(BackendDependenciesForGeneratingTestCase, self).setUp()
        self._config = None
        self._backend_dependencies = None
        self._rasa_config = None
        self._mock_rasa_interpreter = None
        self._mock_rasa_loader = None

    @patch("%s.for_generating.RasaConfig" % dependencies.__name__, autospec=True)
    @patch("%s.abstract_backend_dependencies.BackendConfig" % dependencies.__name__, autospec=True)
    @patch("%s.for_generating.DDDSetLoader" % dependencies.__name__, autospec=True)
    def test_duckling_disabled_with_datetime_predicates_in_ontology_issues_warning(
        self, DddSetLoader, BackendConfig, RasaConfig
    ):
        self.given_mock_backend_config(BackendConfig)
        self.given_mock_ddd_set_loader(DddSetLoader)
        self.given_mock_rasa_config(RasaConfig)
        self.given_ddds(["mock_ddd"])
        self.given_active_ddd_in_config("mock_ddd")
        self.given_rasa_enabled_in("mock_ddd")
        self.given_mocked_ontology_in("mock_ddd")
        self.given_mocked_ontology_has_predicates_of_sort("mock_ddd", DATETIME)
        self.given_duckling_disabled_in_rasa_config()
        self.when_create_backend_dependencies_then_warning_is_issued_matching(
            "DDD 'mock_ddd' contains predicates of the 'datetime' sort, but duckling is disabled. "
            "'datetime' entities won't be recognized in NLU."
        )

    def given_duckling_disabled_in_rasa_config(self):
        self._rasa_config["enable_duckling"] = False

    def when_create_backend_dependencies_then_warning_is_issued_matching(self, expected_warning):
        with patch("{}.for_generating.warnings".format(dependencies.__name__), spec=warnings) as mock_warnings:
            self._create_backend_dependencies()
            mock_warnings.warn.assert_has_calls([call(expected_warning)])

    @patch("%s.for_generating.RasaConfig" % dependencies.__name__, autospec=True)
    @patch("%s.abstract_backend_dependencies.BackendConfig" % dependencies.__name__, autospec=True)
    @patch("%s.for_generating.DDDSetLoader" % dependencies.__name__, autospec=True)
    def test_duckling_disabled_with_integer_predicates_in_ontology_issues_warning(
        self, DddSetLoader, BackendConfig, RasaConfig
    ):
        self.given_mock_backend_config(BackendConfig)
        self.given_mock_ddd_set_loader(DddSetLoader)
        self.given_mock_rasa_config(RasaConfig)
        self.given_ddds(["mock_ddd"])
        self.given_active_ddd_in_config("mock_ddd")
        self.given_rasa_enabled_in("mock_ddd")
        self.given_mocked_ontology_in("mock_ddd")
        self.given_mocked_ontology_has_predicates_of_sort("mock_ddd", INTEGER)
        self.given_duckling_disabled_in_rasa_config()
        self.when_create_backend_dependencies_then_warning_is_issued_matching(
            "DDD 'mock_ddd' contains predicates of the 'integer' sort, but duckling is disabled. "
            "Accuracy for 'integer' entities will be reduced in NLU."
        )

    @patch("%s.for_generating.RasaConfig" % dependencies.__name__, autospec=True)
    @patch("%s.abstract_backend_dependencies.BackendConfig" % dependencies.__name__, autospec=True)
    @patch("%s.for_generating.DDDSetLoader" % dependencies.__name__, autospec=True)
    def test_duckling_disabled_but_no_predicates_of_duckling_sorts_in_ontology_issues_no_warnings(
        self, DddSetLoader, BackendConfig, RasaConfig
    ):
        self.given_mock_backend_config(BackendConfig)
        self.given_mock_ddd_set_loader(DddSetLoader)
        self.given_mock_rasa_config(RasaConfig)
        self.given_ddds(["mock_ddd"])
        self.given_active_ddd_in_config("mock_ddd")
        self.given_rasa_enabled_in("mock_ddd")
        self.given_mocked_ontology_in("mock_ddd")
        self.given_mocked_ontology_has_no_predicates_of_any_sorts("mock_ddd")
        self.given_duckling_disabled_in_rasa_config()
        self.when_create_backend_dependencies_then_no_warnings_are_issued()

    def given_mocked_ontology_has_no_predicates_of_any_sorts(self, ddd):
        self._get_ddd(ddd).ontology.predicates_contain_sort.return_value = False

    def when_create_backend_dependencies_then_no_warnings_are_issued(self):
        with patch("{}.for_generating.warnings".format(dependencies.__name__), spec=warnings) as mock_warnings:
            self._create_backend_dependencies()
            mock_warnings.warn.assert_not_called()

    @patch("%s.for_generating.RasaConfig" % dependencies.__name__, autospec=True)
    @patch("%s.abstract_backend_dependencies.BackendConfig" % dependencies.__name__, autospec=True)
    @patch("%s.for_generating.DDDSetLoader" % dependencies.__name__, autospec=True)
    def test_rasa_disabled_with_datetime_predicates_in_ontology_issues_warning(
        self, DddSetLoader, BackendConfig, RasaConfig
    ):
        self.given_mock_backend_config(BackendConfig)
        self.given_mock_ddd_set_loader(DddSetLoader)
        self.given_mock_rasa_config(RasaConfig)
        self.given_ddds(["mock_ddd"])
        self.given_active_ddd_in_config("mock_ddd")
        self.given_rasa_disabled_in("mock_ddd")
        self.given_mocked_ontology_in("mock_ddd")
        self.given_mocked_ontology_has_predicates_of_sort("mock_ddd", DATETIME)
        self.when_create_backend_dependencies_then_warning_is_issued_matching(
            "DDD 'mock_ddd' contains predicates of the 'datetime' sort, but RASA NLU is disabled. "
            "'datetime' entities won't be recognized in NLU."
        )

    @patch("%s.for_generating.RasaConfig" % dependencies.__name__, autospec=True)
    @patch("%s.abstract_backend_dependencies.BackendConfig" % dependencies.__name__, autospec=True)
    @patch("%s.for_generating.DDDSetLoader" % dependencies.__name__, autospec=True)
    def test_rasa_disabled_but_no_predicates_of_datetime_sort_in_ontology_issues_no_warnings(
        self, DddSetLoader, BackendConfig, RasaConfig
    ):
        self.given_mock_backend_config(BackendConfig)
        self.given_mock_ddd_set_loader(DddSetLoader)
        self.given_mock_rasa_config(RasaConfig)
        self.given_ddds(["mock_ddd"])
        self.given_active_ddd_in_config("mock_ddd")
        self.given_rasa_disabled_in("mock_ddd")
        self.given_mocked_ontology_in("mock_ddd")
        self.given_mocked_ontology_has_no_predicates_of_any_sorts("mock_ddd")
        self.when_create_backend_dependencies_then_no_warnings_are_issued()

    def _create_backend_dependencies(self):
        self._backend_dependencies = BackendDependenciesForGenerating(self._mock_args)
